//*** The code for the simulations of the article 
//*** "Agent-based model for studying the effect of solid stress and nutrient supply on tumor growth"
//*** by Maxim Kuznetsov and Andrey Kolobov
//*** submitted to Biomechanics and Modeling in Mechanobiology
//*** 2022
//*** This code works fine in Dev-C++, version 5.5.2 
//*** Note that some small parts of the code may need to be revised depending on the utilized software 

//***********************************************************************************************************************************************************
//********************************* INTRODUCTION ************************************************************************************************************
//***********************************************************************************************************************************************************

#include <stdio.h>
#include <tchar.h>
#include <windows.h>
#include <ctime>
#include <iostream>
#include <cmath>
#include <math.h>
#include <conio.h>
#include <fstream>
using namespace std;
#define PI 3.14159265359

// note that here another length normalization is used than in the article (10^(-4) meters instead of micrometers)
// it is kept as was in order to reproduce close results -- alteration of normalization yields a bit different behaviour due to numerical effects
// this normalization affects parameters r0 (and those defined via it), K (and the way of tau definition via it), KCT, KCN, s0N, Dn

double r0 = 0.05; // initial (and maximum) cell radius // 0.05 is analogical to 5. in article
double kT = 80.; // coefficient of stress function for tumor cells
double kN = 80.; // ... normal cells
double kC = 80.; // ... between cell and capsule
double kTN = (kT + kN) / 2.; // ... between tumor and normal cells (actual if there is no tumor capsule)

double l0 = (0.15*r0); // normal bond length (yielding zero stress)
double lmax = r0; // maximum bond length
double lbond = lmax; // on which distance bond between cells is formed

// used in stress function
double c0 = (2. * r0)*(2. * r0)*(2. * r0) / ((2. * r0 + l0)*(2. * r0 + l0)*(2. * r0 + l0));
double ccr = (2. * r0)*(2. * r0)*(2. * r0) / ((2. * r0 + lmax)*(2. * r0 + lmax)*(2. * r0 + lmax));
double cinf = (2. * r0)*(2. * r0)*(2. * r0) / ((2. * r0 - l0)*(2. * r0 - l0)*(2. * r0 - l0));

double Scr = 15.; // critical stress at which cell divergence stops
double B = 0.03; // maximum cell growth rate (in reverse hours)
double v = r0 / (log(2.) / B); // maximum speed of divergence of a paired tumor cell
double K = 0.000001; // tissue conductivity  // 0.000001 is analogical to 0.01 in article

int tumor_capsule = 1; // 0 -- no tumor capsule, 1 -- soft
int norm_capsule = 1; // 0 -- no normal capsule, 1 -- soft, 2 -- rigid immobile 
int layers = 5; // number of approximate normal cell layers around tumor cells (5, 10, 15, 20 -- to obtain article's parameters of initial normal tissue radius)
int Nutr = 0; // nutrient is accounted for

double KCT = 0.0005; // coefficient of tumor capsule speed of growth // 0.0005 is analogical to 5. in article
double KCN = 0.00005; // coefficient of normal capsule speed of growth // 0.00005 is analogical to 0.5 in article
double s0N = 10.; // minimal pressure to keep normal capsule static // 10. is analogical to 0.1 in article
double M = 0.01; // tumor cell shrink/death rate
double Mn = 0.1; // normal cell shrink/death rate
double rmin = (0.5*r0); // radius, at which cells die

// nutrient
double P = 4.;// rate of nutrient inflow
double p = P / (r0*r0);
double Dn = 0.3; // diffusion coefficient // 0.3 is analogical to 3000. in article
double QN = 0.5; // rate of nutrient consumption by a normal cell and a quiescent tumor cell
double QT = 25.; // maximum rate of nutrient consumption by one tumor cell due to proliferation
double qT = QT / (r0*r0);
double qN = QN / (r0*r0);
double nst = 0.005; // Michaelis-Menten parameter for nutrient consumption
double nq = 0.1; // nutrient level below which tumor cells become quiescent
double nd = 0.01; // nutrient level below which cells shrink and die

// technical parameters

double tau, tauMax, tauMin; // time step
int Adjust_tau = 1; // if 1, time step is adjusted during simulation (decreases when bonds length change rapidly)
double dR = r0 / 1000.; // initial tiny displacement of dividing cells
double Tpot;// period of checking for potential bonds formation
double Tbond;// period of checking for bonds formation
double lpot = 5.*lbond; // length at which potential bond is formed
double lpotbr = 7.*lbond; // length at which potential bond is not accounted for anymore

double Smax = 10.*Scr; // max force value accounted for in motion, to avoid technical difficulties

double volume_frequency = 0.1; // how often to write down tumor volume, capsule radii, number of tumor cells
int OutputFlg = 1; // output files with cells and bonds information will be performed
double output_frequency = 1.; // how often to generate output files with cells and bonds information
int nofo = 0; // no final output, if 1

double t = 0.; // time counter
double tv = volume_frequency; // counter for tumor volume output
double to = output_frequency; // counter for data output
double tnl = 0.; // counter for bond formation
double tnlr = 0.; // counter for potential bond formation

double T = 10.*24.*365.; // time to stop (24.*365. is 1 year)
long int Nstop = 1000000000; // program will stop if number of pairs of cells is this

double Lprev; // can be used to control time step dynamically
double WorstShrink; // can be used to control time step dynamically

int N; // estimated number of cells during initial generation
double RadiusSq; // initial radius of tissue

int N_Cells = 2; // cells counter
int N_Alive_Tumor_Cells = 1;
int N_Cells_Init;
int N_Bonds = 1; // bonds counter
int N_BondsOld, N_BondsOld2;
int CellsMaxN, CellsMaxNold; // max number of cells (can and will be increased when needed as well as other parameters below)
int BondsMaxN, BondsMaxNold; // max number of bonds
int MaxBondsForCell = 30; // maximum number of bonds for one cell
int MaxBondsForCellold;
int MaxBondsForCapsule = 3000; // maximum number of bonds for one capsule
int MaxBondsForCapsuleold;

// functions

double Stress(double ll, double kk); // solid stress in dependence of distance between cells

void Output(); // write down information about cells and bonds in files
void VolumeOutput(); // write down tumor volume, radii of capsules and number of cell pairs
void CellKill(int CK); // kill cell with indicated number

					   // to get memory when needed
void IncreaseCellsMaxN();
int IncreaseBondsMaxN(int li);
int IL = 0; // to get value from IncreaseBondsMaxN when actually it is not needed
void IncreaseMaxBondsForCell();
void IncreaseMaxBondsForCapsule();

// for debugging

int debugd = 9999999; // from this output index d output will be generated every time step (but if tau is being adjusted, not strictly every, but often)
int debugd2 = 9999999; // additionally -- you can record data about every action
int outputEveryAction = 0; // if 1, cells and bonds information is recorded after every action (cells movement, bonds redefinition, etc.)

						   // arrays

double *PressureOnCell; // [CellsMaxN]; // pressure exerted on tumor cell -- to define whether it should grow and at what rate
double **Cells; // [CellsMaxN][4] // 0 -- x; 1 -- y; 2 -- radius; 3 -- fraction of considered area (0.5-1)
int *CellsTypes; // [CellsMaxN]  // type of cell: 0 -- tumor, 1 -- normal, 2 -- norm capsule; 3 -- tumor capsule; -1 -- dead
int *CellMoveMode; // [CellsMaxN] // 1 -- cells move apart in the process of division; 2 -- their radii grow; 3 -- they shrink until r=rmin, then die

double **Nutrient, **NutrientBu; // [CellsMaxN][4] // level of nutrient (if normal cells exist); 0 -- x; 1 -- y; 2 -- level; 3 -- coefficient for rate of growth of supply;
int *NutrCells; // [CellsMaxN] // for convenience -- list of active grid positions with type: -1 -- inactive; 0 -- tumor; 1 -- normal;
int *NormalCellsToCheck, *NormalCellsToCheck2; // [N_Cells_Init] -- to help checking which normal cells supply nutrient
int StopIt; // to help checking which normal cells supply nutrient

double **Bonds; // [BondsMaxN][6] // 0 -- length; 1 -- point of beginning x; 2 -- point of beginning y; 3 -- point of end x; 4 -- point of end y; 5 -- coefficient of stress (non-unity for closed bonds)
int **BondsCells; // [BondsMaxN][3] // indices of bonds with indication of cells: 0 -- beginning; 1 -- end; 
				  //2 -- and type of bond: 0 -- broken, 1 -- open between cells, 2 -- closed, goes through pair of 1st cell, 3 -- closed, goes through pair of 2st cell, 4 -- closed, goes through pairs of both cells
				  // 5 -- open bond with norm capsule, 6 -- closed bond with norm capsule, 
				  // 7 -- open bond of tumor cell with tumor capsule, 8 -- closed bond of tumor cell with tumor capsule, 9 -- bond between dividing cells
int *BondsCorr; // [N_Bonds][2] // for optimization when broken bonds are deleted 

int ***CellsConnectedWith; // [CellsMaxN][MaxBondsForCell][2]; // 0 -- indices of cells/capsules with which each cell is connected; 1 -- indices of corresponding bonds
int ***CapsulesConnectedWith; // [2][MaxBondsForCapsule][2]; // 0 -- indices of cells with which capsule is connected; 1 -- indices of corresponding bonds
int *TumorCellsPair; // [CellsMaxN] // indices of dividing brother-cell for tumor cells 
int **CellsPotConnected; // [CellsMaxN][5*MaxBondsForCell]; // indices of cells that should be checked for new bonds with each cell
					  // note --  it would be convenient to just create CellsMaxN x CellsMaxN matrix of connection, but this will not work for great N due to the lack of memory
int l, ll, lll, llll; // counters for these arrays

					  // auxiliary arrays for increase of size
double *PressureOnCellNew;
double **CellsNew, **BondsNew, **NutrientNew;
int *CellsTypesNew, *CellMoveModeNew, *TumorCellsPairNew, *NutrCellsNew;
int **BondsCellsNew, **CellsPotConnectedNew;
int ***CellsConnectedWithNew, ***CapsulesConnectedWithNew;

// auxiliary variables

double TumorVolume, ProlTumorVolume, QuiesTumorVolume, DyingTumorVolume;

double DivDir; // direction of division
double DivDir0 = 0.* PI / 180.; // initial direction of division

double ScrGrow; // critical stress for growing cells -- depends on their radii and division bond length

double Force, ForceX, ForceY; // force acting on a cell and its projections
double ForcePr, ForcePrX, ForcePrY; // force acting on a cell and its projections -- without restriction
double MaxForce; // helps to indicate the direction of division
int DontAccount; // flag that bond should not be accounted for
double ltest, ltest2; // length between cells, used to check whether they should be connected
double F[2]; // total force applied to cell in directions: 0 -- x, 1 -- y
double kF; // coefficient for calculating force from stress

double cc; // to use in stress function
double kB, kb, kM, kMn; // coefficients of growth and shrinkage
double krot; // for correction of cells position after rotation
double begx, begy, endx, endy; // beginning and end of bonds
int CellA, CellB, CellC, CellD, CellE, CellAA, CellBB, CellCC, BondA; // cells and bond indices
double D, L, alpha, cosbeta, LH, fS, fSinit, I, JoinX, JoinY, CenterX, CenterY, Torque; // cell area, moment of inertia of joined diving cells and others auxilliary variables
double dXboth, dYboth, dPhi, dRapartA, dXapartA, dYapartA, dRapartB, dXapartB, dYapartB; // displacements of cells due to various reasons
double l1test, l2test, l3test, l4test, l5test; // intermediate bond data
double dx; // distance between nutrient grid points;
double Jg; // flow of nutrient;
int NCalcNutr = 1;
double NCalcNutrNorm = 0.00002; // to define number of nutrient field calculations in a single time step
int Achtung = 0; // to prevent numerical problems with nutrient calculation
				 // for output

int d = 0; // for serial numbers of the files with distributions of varibles
string str;
int nn;
char * char_array;
char File_name_cells[80], File_name_Bonds[80], num[10];
FILE * out;
FILE * volume;

// main part
int main()
{
	{
		// read parameters from the folder name
		char buffer[MAX_PATH];
		GetModuleFileNameA(NULL, buffer, MAX_PATH);
		string str(buffer);

		// Dn
		size_t pos_Dn = str.find("Dn=");
		if (pos_Dn != -1)
		{
			string str_Dn = str.substr(pos_Dn + 3);
			Dn = atof(str_Dn.c_str());
			cout << "Dn is " << Dn << "\n";
		}

		// P
		size_t pos_P = str.find("P=");
		if (pos_P != -1)
		{
			string str_P = str.substr(pos_P + 2);
			P = atof(str_P.c_str());
			cout << "P is " << P << "\n";
			p = P / (r0*r0);
		}

		// kT
		size_t pos_ktt = str.find("kT=");
		if (pos_ktt != -1)
		{
			string str_ktt = str.substr(pos_ktt + 3);
			kT = atof(str_ktt.c_str());
			cout << "kT is " << kT << "\n";
		}

		// kN
		size_t pos_knn = str.find("kN=");
		if (pos_knn != -1)
		{
			string str_knn = str.substr(pos_knn + 3);
			kN = atof(str_knn.c_str());
			cout << "kN is " << kN << "\n";
		}

		// kTN
		size_t pos_ktn = str.find("kTN=");
		if (pos_ktn != -1)
		{
			string str_ktn = str.substr(pos_ktn + 4);
			kTN = atof(str_ktn.c_str());
			cout << "kTN is " << kTN << "\n";
		}
		else
			kTN = (kT + kN) / 2.;

		// Scr
		size_t pos_Scr = str.find("Scr=");
		if (pos_Scr != -1)
		{
			string str_Scr = str.substr(pos_Scr + 4);
			Scr = atof(str_Scr.c_str());
			cout << "Scr is " << Scr << "\n";
		}

		// DivDir0
		size_t pos_DD0 = str.find("DD0=");
		if (pos_DD0 != -1)
		{
			string str_DD0 = str.substr(pos_DD0 + 4);
			DivDir0 = atof(str_DD0.c_str());
			cout << "DivDir0 is " << DivDir0 << "\n";
		}

		// K
		size_t pos_K = str.find("K=");
		if (pos_K != -1)
		{
			string str_K = str.substr(pos_K + 2);
			K = atof(str_K.c_str());
			cout << "K is " << K << "\n";
		}

		// KCN
		size_t pos_KCN = str.find("KCN=");
		if (pos_KCN != -1)
		{
			string str_KCN = str.substr(pos_KCN + 8);
			KCN = atof(str_KCN.c_str());
			cout << "KCN is " << KCN << "\n";
		}

		// KCT
		size_t pos_KCT = str.find("KCT=");
		if (pos_KCT != -1)
		{
			string str_KCT = str.substr(pos_KCT + 8);
			KCT = atof(str_KCT.c_str());
			cout << "KCT is " << KCT << "\n";
		}

		tauMax = 4.e-7 / K;
		tauMin = min(0.01, 4.e-8 / K);
		tau = tauMin;
		Tbond = tauMax;
		Tpot = 70.*tauMax;
		cout << "tauMax is " << tauMax << "\n";

		// layers
		size_t pos_layers = str.find("layers=");
		if (pos_layers != -1)
		{
			string str_layers = str.substr(pos_layers + 7);
			layers = atoi(str_layers.c_str());
			cout << "layers is " << layers << "\n";
		}

		if (layers > 0)
			layers = max(3, layers + 2);

		if (layers == 0)
			norm_capsule = 0;

		// no tumor capsule
		size_t pos_ntc = str.find("ntc");
		if (pos_ntc != -1)
		{
			tumor_capsule = 0;
			cout << "No tumor capsule\n";
		}

		// no normal capsule
		size_t pos_nc = str.find("nnc");
		if (pos_nc != -1)
		{
			norm_capsule = 0;
			cout << "No normal capsule\n";
		}

		// no nutrient
		size_t pos_gl = str.find("NoNutr");
		if (pos_gl != -1)
		{
			Nutr = 0;
			cout << "No nutrient\n";
		}

		// ridig normal capsule
		size_t pos_hc = str.find("hnc");
		if (pos_hc != -1)
		{
			norm_capsule = 2;
			cout << "Hard normal capsule\n";
		}

		// no final output
		size_t pos_nofo = str.find("nofo");
		if (pos_nofo != -1)
		{
			nofo = 1;
			cout << "No final output\n";
		}

		// number of cell pairs to stop
		size_t pos_Nstop = str.find("Nstop=");
		if (pos_Nstop != -1)
		{
			string str_Nstop = str.substr(pos_Nstop + 6);
			Nstop = atof(str_Nstop.c_str());
			cout << "Nstop is " << Nstop << "\n";
		}

		// time to stop		
		size_t pos_T = str.find("T=");
		if (pos_T != -1)
		{
			string str_T = str.substr(pos_T + 2);
			T = atof(str_T.c_str());
			cout << "T is " << T << "\n";
		}

		//output_frequency
		size_t pos_OF = str.find("OF=");
		if (pos_OF != -1)
		{
			string str_OF = str.substr(pos_OF + 3);
			output_frequency = atof(str_OF.c_str());
			cout << "output_frequency is " << output_frequency << "\n";
			to = output_frequency;
		}

		size_t pos_f = str.find("hour");
		if (pos_f != -1)
		{
			output_frequency = 1.;
			cout << "output_frequency is " << output_frequency << "\n";
		}

		size_t pos_f1 = str.find("10min");
		if (pos_f1 != -1)
		{
			output_frequency = (1. / 6.);
			cout << "output_frequency is " << output_frequency << "\n";
		}

		size_t pos_f2 = str.find("5min");
		if (pos_f2 != -1)
		{
			output_frequency = (1. / 12.);
			cout << "output_frequency is " << output_frequency << "\n";
		}

		size_t pos_f3 = str.find("1min");
		if (pos_f3 != -1)
		{
			output_frequency = (1. / 60.);
			cout << "output_frequency is " << output_frequency << "\n";
		}

		size_t pos_f4 = str.find("no output");
		if (pos_f4 != -1)
		{
			OutputFlg = 0;
			to = output_frequency;
		}

		// debugging -- frequent output from indicated d
		size_t pos_dd = str.find("debug=");
		if (pos_dd != -1)
		{
			string str_dd = str.substr(pos_dd + 6);
			debugd = atoi(str_dd.c_str());
			cout << "I will output every tau from " << debugd << "\n";
		}

		// debugging -- output after every action from indicated d
		size_t pos_dd2 = str.find("debug2=");
		if (pos_dd2 != -1)
		{
			string str_dd2 = str.substr(pos_dd2 + 7);
			debugd2 = atoi(str_dd2.c_str());
			cout << "I will output every action from " << debugd2 << "\n";
		}
	}

	// initialize arrays
	N = (1 + (6 * layers*(layers + 1) / 2));
	CellsMaxN = max(4 * N, 1000);
	BondsMaxN = max(24 * N, 6000);

	PressureOnCell = new double[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		PressureOnCell[i] = 0.;

	Cells = new double*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		Cells[i] = new double[4];

	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 3; j++)
			Cells[i][j] = 0.;

	Nutrient = new double*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		Nutrient[i] = new double[4];
	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		Nutrient[i][0] = 0.;
		Nutrient[i][1] = 0.;
		Nutrient[i][2] = 1.;
		Nutrient[i][3] = 1.;
	}

	NutrientBu = new double*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		NutrientBu[i] = new double[4];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 3; j++)
			NutrientBu[i][j] = 0.;

	NutrCells = new int[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		NutrCells[i] = -1;

	CellsTypes = new int[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellsTypes[i] = -1;

	CellMoveMode = new int[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellMoveMode[i] = 1;

	Bonds = new double*[BondsMaxN];
	for (int i = 0; i <= BondsMaxN - 1; i++)
		Bonds[i] = new double[6];

	for (int i = 0; i <= BondsMaxN - 1; i++)
		for (int j = 0; j <= 5; j++)
			Bonds[i][j] = 0.;

	BondsCells = new int*[BondsMaxN];
	for (int i = 0; i <= BondsMaxN - 1; i++)
		BondsCells[i] = new int[3];

	for (int i = 0; i <= BondsMaxN - 1; i++)
		for (int j = 0; j <= 2; j++)
			BondsCells[i][j] = -1;

	CellsConnectedWith = new int**[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		CellsConnectedWith[i] = new int*[MaxBondsForCell];
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			CellsConnectedWith[i][j] = new int[2];
	}
	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			for (int jj = 0; jj <= 1; jj++)
				CellsConnectedWith[i][j][jj] = -1;

	CapsulesConnectedWith = new int**[2];
	for (int i = 0; i <= 1; i++)
	{
		CapsulesConnectedWith[i] = new int*[MaxBondsForCapsule];
		for (int j = 0; j <= MaxBondsForCapsule - 1; j++)
			CapsulesConnectedWith[i][j] = new int[2];
	}
	for (int i = 0; i <= 1; i++)
		for (int j = 0; j <= MaxBondsForCapsule - 1; j++)
			for (int jj = 0; jj <= 1; jj++)
				CapsulesConnectedWith[i][j][jj] = -1;

	TumorCellsPair = new int[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		TumorCellsPair[i] = -1;

	CellsPotConnected = new int*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellsPotConnected[i] = new int[5 * MaxBondsForCell];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 5 * MaxBondsForCell - 1; j++)
			CellsPotConnected[i][j] = -1;

	// file for tumor volume
	if ((volume = fopen("!Volume.txt", "wt")) == NULL)
		fprintf(stderr, "Cannot open file for tumor volume.txt\n\n");

	fprintf(volume, "t\t\tVolume\t\tRtumcap\t\tRnormcap\t\tProl\t\t\tQuies\t\t\tDying\t\tTumorCells\n\n");
	fflush(volume);

	// INITIAL CONDITIONS

	// 0 "cell" is normal tissue capsule
	Cells[0][0] = 0.; // x
	Cells[0][1] = 0.; // y
	if (norm_capsule > 0)
		CellsTypes[0] = 2;
	else
		CellsTypes[0] = -1;
	// its radius ([2]) will be defined later
	Cells[0][3] = 1.; // its area fraction is not needed but just in case it is set to 1

					  // 1st "cell" is tumor capsule
	Cells[1][0] = 0.; // x
	Cells[1][1] = 0.; // y
	if (tumor_capsule > 0)
	{
		CellsTypes[1] = 3;
		Cells[1][2] = r0 + l0 / 2.;
	}
	else
		CellsTypes[1] = -1;
	Cells[1][3] = 1.;

	//center tumor cell already begins with dividing, so there are two of them
	DivDir = DivDir0;

	Cells[2][0] = dR*cos(DivDir); // x
	Cells[2][1] = dR*sin(DivDir); // y
	Cells[2][2] = r0; // radius
	CellsTypes[2] = 0;
	N_Cells++;

	Cells[3][0] = -dR*cos(DivDir); // x
	Cells[3][1] = -dR*sin(DivDir); // y
	Cells[3][2] = r0; // radius
	CellsTypes[3] = 0;
	N_Cells++;

	// their initial area, will be used further for all cells that have begun to divide
	D = sqrt((Cells[2][0] - Cells[3][0])*(Cells[2][0] - Cells[3][0]) + (Cells[2][1] - Cells[3][1])*(Cells[2][1] - Cells[3][1]));
	alpha = 2 * acos(D / (2.*Cells[2][2]));
	fSinit = (2 * PI - alpha + sin(alpha)) / (2 * PI);

	Cells[2][3] = fSinit;
	Cells[3][3] = fSinit;

	// dividing bond between these cells
	BondsCells[N_Bonds][0] = 2;
	BondsCells[N_Bonds][1] = 3;
	BondsCells[N_Bonds][2] = 9;

	Bonds[N_Bonds][0] = -2.*Cells[2][2] + 2.*dR; // length
	Bonds[N_Bonds][1] = Cells[2][0] - Cells[2][2] * cos(DivDir); // beginning x
	Bonds[N_Bonds][2] = Cells[2][1] - Cells[2][2] * sin(DivDir); // beginning y
	Bonds[N_Bonds][3] = Cells[3][0] + Cells[3][2] * cos(DivDir); // end x
	Bonds[N_Bonds][4] = Cells[3][1] + Cells[3][2] * sin(DivDir); // end y
	Bonds[N_Bonds][5] = 1.; // coefficient of force

	CellsConnectedWith[2][0][0] = 3;
	CellsConnectedWith[3][0][0] = 2;

	CellsConnectedWith[2][0][1] = N_Bonds;
	CellsConnectedWith[3][0][1] = N_Bonds;

	N_Bonds++;

	TumorCellsPair[2] = 3;
	TumorCellsPair[3] = 2;

	// layers of normal cells
	for (int i = 1; i <= layers; i++)
	{
		int CF = (1 + (6 * i*(i - 1) / 2)); // index of the first cell in this layer

		for (int j = 0; j <= 5; j++)
		{
			Cells[CF + j*i + 3][0] = i*(2 * r0 + l0)*cos(j*PI / 3.); // x
			Cells[CF + j*i + 3][1] = i*(2 * r0 + l0)*sin(j*PI / 3.); // y
			Cells[CF + j*i + 3][2] = r0; // radius 
			Cells[CF + j*i + 3][3] = 1.; // area fraction

			N_Cells++;
			if (N_Cells >= CellsMaxN - 10)
				IncreaseCellsMaxN();
		}

		for (int kk = 1; kk <= i - 1; kk++)
		{
			double CellX = (1.*i - 0.5*kk)*(2 * r0 + l0);
			double CellY = kk*(sqrt(3.) / 2.)*(2 * r0 + l0);
			double CellR = sqrt(CellX*CellX + CellY*CellY);
			double CellPhi = atan(CellY / CellX);

			for (int j = 0; j <= 5; j++)
			{
				Cells[CF + kk + j*i + 3][0] = CellR*cos(CellPhi + j*PI / 3.); // x
				Cells[CF + kk + j*i + 3][1] = CellR*sin(CellPhi + j*PI / 3.); // y
				Cells[CF + kk + j*i + 3][2] = r0;
				Cells[CF + kk + j*i + 3][3] = 1.;

				N_Cells++;
				if (N_Cells >= CellsMaxN - 10)
					IncreaseCellsMaxN();
			}
		}
	}

	N_Cells_Init = N_Cells;

	NormalCellsToCheck = new int[N_Cells_Init];
	for (int i = 0; i <= N_Cells_Init - 1; i++)
		NormalCellsToCheck[i] = -1;

	NormalCellsToCheck2 = new int[N_Cells_Init];
	for (int i = 0; i <= N_Cells_Init - 1; i++)
		NormalCellsToCheck2[i] = -1;

	// BONDS between cells + tumor capsule with normal cells
	for (int i = 2 - tumor_capsule; i <= N_Cells - 1; i++)
	{
		if (!(tumor_capsule == 1 && (i == 2 || i == 3)))
			for (int j = max(4, i + 1); j <= N_Cells - 1; j++)
			{
				ltest = sqrt((Cells[j][0] - Cells[i][0])*(Cells[j][0] - Cells[i][0]) + (Cells[j][1] - Cells[i][1])*(Cells[j][1] - Cells[i][1])) - Cells[j][2] - Cells[i][2];

				if (ltest < 1.01*lbond) // due to numerical errors it is better to do like this, otherwise there can be not all bonds with length l0
				{
					// connect cells

					if (i == 1) // it'a a tumor capsule
					{
						l = 0;
						while (CapsulesConnectedWith[i][l][0] != -1)
							l++;
						if (l >= MaxBondsForCapsule - 10)
							IncreaseMaxBondsForCapsule();
						CapsulesConnectedWith[i][l][0] = j;
						CapsulesConnectedWith[i][l][1] = N_Bonds;

						BondsCells[N_Bonds][0] = j;
						BondsCells[N_Bonds][1] = i;
					}
					else //it's a cell
					{
						l = 0;
						while (CellsConnectedWith[i][l][0] != -1)
							l++;
						if (l >= MaxBondsForCell - 10)
							IncreaseMaxBondsForCell();
						CellsConnectedWith[i][l][0] = j;
						CellsConnectedWith[i][l][1] = N_Bonds;

						BondsCells[N_Bonds][0] = j;
						BondsCells[N_Bonds][1] = i;
					}

					l = 0;
					while (CellsConnectedWith[j][l][0] != -1)
						l++;
					if (l >= MaxBondsForCell - 10)
						IncreaseMaxBondsForCell();
					CellsConnectedWith[j][l][0] = i;
					CellsConnectedWith[j][l][1] = N_Bonds;

					// define type of bond (it's yet open bond between cells, or tumor capsule with normal cell)
					BondsCells[N_Bonds][2] = 1;

					Bonds[N_Bonds][0] = ltest;
					Bonds[N_Bonds][1] = Cells[j][0] + (Cells[i][0] - Cells[j][0])*Cells[j][2] / (Cells[j][2] + Cells[i][2] + Bonds[N_Bonds][0]);
					Bonds[N_Bonds][2] = Cells[j][1] + (Cells[i][1] - Cells[j][1])*Cells[j][2] / (Cells[j][2] + Cells[i][2] + Bonds[N_Bonds][0]);
					Bonds[N_Bonds][3] = Cells[i][0] + (Cells[j][0] - Cells[i][0])*Cells[i][2] / (Cells[j][2] + Cells[i][2] + Bonds[N_Bonds][0]);
					Bonds[N_Bonds][4] = Cells[i][1] + (Cells[j][1] - Cells[i][1])*Cells[i][2] / (Cells[j][2] + Cells[i][2] + Bonds[N_Bonds][0]);

					// for tumor cell -- if bond goes through dividing brother cell, this bond is closed and has coefficient of stress lower than unity
					// now only one tumor cell can participate in a bond (2-3 bond is internal and already accounted for)

					if (CellsTypes[i] == 0) // tumor cell
					{
						begx = Bonds[N_Bonds][3];
						begy = Bonds[N_Bonds][4];
						endx = Bonds[N_Bonds][1];
						endy = Bonds[N_Bonds][2];

						L = sqrt((begx - Cells[TumorCellsPair[i]][0])*(begx - Cells[TumorCellsPair[i]][0]) + (begy - Cells[TumorCellsPair[i]][1])*(begy - Cells[TumorCellsPair[i]][1]));

						alpha = acos(((endx - begx)*(Cells[TumorCellsPair[i]][0] - begx) + (endy - begy)*(Cells[TumorCellsPair[i]][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
						if (alpha < PI / 2.)
							L = L*sin(alpha);

						if (L < Cells[TumorCellsPair[i]][2])
						{
							cosbeta = ((Bonds[N_Bonds][1] - Bonds[N_Bonds][3])*(Cells[TumorCellsPair[i]][0] - Cells[i][0]) + (Bonds[N_Bonds][2] - Bonds[N_Bonds][4])*(Cells[TumorCellsPair[i]][1] - Cells[i][1])) / (ltest*D);

							if (D*D*(cosbeta*cosbeta - 1.) + Cells[TumorCellsPair[i]][2] * Cells[TumorCellsPair[i]][2] > 0.) // just in case, numerical errors are possible
								LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[TumorCellsPair[i]][2] * Cells[TumorCellsPair[i]][2]) - Cells[i][2]; // "hidden" length of bond
							else
								LH = 0.;

							if (LH > 0.) // just in case, numerical errors are possible
							{
								Bonds[N_Bonds][0] = Bonds[N_Bonds][0] - LH; // effective length 
								Bonds[N_Bonds][5] = (1. + ((1. - 2.*fSinit) / fSinit) *(LH / D));
								BondsCells[N_Bonds][2] = 3;
							}
						}
						else
							Bonds[N_Bonds][5] = 1.;
					}
					else
						Bonds[N_Bonds][5] = 1.;

					N_Bonds++;
					if (N_Bonds >= BondsMaxN - 10)
						IL = IncreaseBondsMaxN(IL);
				}
			}
	}

	RadiusSq = 1.001*(Cells[N_Cells - 1][0] * Cells[N_Cells - 1][0] + Cells[N_Cells - 1][1] * Cells[N_Cells - 1][1]);

	for (int i = N_Cells - 2; i >= (1 + (6 * layers*(layers - 1) / 2)); i--)
		RadiusSq = min(RadiusSq, 1.001*(Cells[i][0] * Cells[i][0] + Cells[i][1] * Cells[i][1]));

	// define normal cells
	for (int i = 4; i <= N_Cells - 1; i++)
	{
		CellsTypes[i] = 1;
		NutrCells[i] = 1; // will be maintained
	}

	// make tissue "round"
	for (int i = 10; i <= N_Cells - 1; i++)
		if (sqrt(Cells[i][0] * Cells[i][0] + Cells[i][1] * Cells[i][1]) + r0 > sqrt(RadiusSq))
			CellKill(i);

	// Bonds with normal capsule
	Cells[0][2] = 0.; // define its radius
	if (norm_capsule > 0)
		for (int i = 2; i <= N_Cells - 1; i++)
		{
			if (CellsTypes[i] == 1 || CellsTypes[i] == 0)
				Cells[0][2] = max(Cells[0][2], sqrt(Cells[i][0] * Cells[i][0] + Cells[i][1] * Cells[i][1]) + r0 + l0);
		}

	// connect cells to capsules
	if (tumor_capsule + norm_capsule > 0)
		for (int iii = 1 - min(1, norm_capsule); iii <= tumor_capsule; iii++)
		{
			if (iii == 1)
			{
				l = 2;
				lll = 3;
			}
			else
			{
				if (layers == 0)
					l = 2;
				else
					l = 4;

				lll = N_Cells - 1;
			}

			for (int j = l; j <= lll; j++)
			{
				ltest = Cells[iii][2] - sqrt((Cells[j][0])*(Cells[j][0]) + (Cells[j][1])*(Cells[j][1])) - Cells[j][2];

				if (ltest < 1.01*lbond) // due to numerical errors it is better to multiply by coefficient slightly greater than 1
				{
					l = 0;
					while (CapsulesConnectedWith[iii][l][0] != -1)
						l++;
					if (l >= MaxBondsForCapsule - 10)
						IncreaseMaxBondsForCapsule();
					CapsulesConnectedWith[iii][l][0] = j;
					CapsulesConnectedWith[iii][l][1] = N_Bonds;

					l = 0;
					while (CellsConnectedWith[j][l][0] != -1)
						l++;
					if (l >= MaxBondsForCell - 10)
						IncreaseMaxBondsForCell();
					CellsConnectedWith[j][l][0] = iii;
					CellsConnectedWith[j][l][1] = N_Bonds;

					BondsCells[N_Bonds][0] = j;
					BondsCells[N_Bonds][1] = iii;

					// define type of bond
					if (iii == 0)
						BondsCells[N_Bonds][2] = 5;
					if (iii == 1)
						BondsCells[N_Bonds][2] = 7;

					Bonds[N_Bonds][0] = ltest;
					Bonds[N_Bonds][1] = Cells[j][0] + Cells[j][2] * Cells[j][0] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
					Bonds[N_Bonds][2] = Cells[j][1] + Cells[j][2] * Cells[j][1] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
					Bonds[N_Bonds][3] = Cells[j][0] + (Cells[j][2] + Bonds[N_Bonds][0]) * Cells[j][0] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
					Bonds[N_Bonds][4] = Cells[j][1] + (Cells[j][2] + Bonds[N_Bonds][0]) * Cells[j][1] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
					if (iii == 0)
						Bonds[N_Bonds][5] = 1.;
					if (iii == 1)
						Bonds[N_Bonds][5] = 3.;

					N_Bonds++;
					// closed bonds are yet impossible
					if (N_Bonds >= BondsMaxN - 10)
						IL = IncreaseBondsMaxN(IL);
				}
			}
		}

	// which potential bonds should be monitored (no potential bonds for tumor capsule here yet)
	for (int i = 2; i <= N_Cells - 1; i++)
	{
		l = 0;
		while (CellsConnectedWith[i][l][0] != -1)
		{
			ll = 0;
			while (CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0] != -1)
			{
				if (CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0] != i)
				{
					lll = 0;

					DontAccount = 0;

					if (CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0] == 0)
						DontAccount = 1;
					else if (CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0] == 1)
						DontAccount = 1;

					else
					{
						while (CellsConnectedWith[i][lll][0] != -1)
						{
							if (CellsConnectedWith[i][lll][0] == CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0])
								DontAccount = 1;
							lll++;
						}

						lll = 0;
						while (CellsPotConnected[i][lll] != -1)
						{
							if (CellsPotConnected[i][lll] == CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0])
								DontAccount = 1;
							lll++;
						}
					}

					if (DontAccount == 0 && !(tumor_capsule == 1 && CellsTypes[i] + CellsTypes[CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0]] == 1))
					{
						if (lll >= 5 * MaxBondsForCell - 10)
							IncreaseMaxBondsForCell();
						CellsPotConnected[i][lll] = CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0];

						// symmetric record
						lll = 0;
						while (CellsPotConnected[CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0]][lll] != -1)
							lll++;
						if (lll >= 5 * MaxBondsForCell - 10)
							IncreaseMaxBondsForCell();
						CellsPotConnected[CellsConnectedWith[CellsConnectedWith[i][l][0]][ll][0]][lll] = i;
					}
				}
				ll++;
			}
			l++;
		}
	}

	// initial nutrient level
	if (layers > 0 && Nutr == 1) // if normal cells exist
	{
		for (int i = 2; i <= N_Cells - 1; i++) // define grid points
		{
			if (CellsTypes[i] == 1) // normal cell
			{
				//NutrCells[i] = 1; // maintained
				Nutrient[i][0] = Cells[i][0]; // x
				Nutrient[i][1] = Cells[i][1]; // y
				Nutrient[i][2] = 1.; // level
				Nutrient[i][3] = 1.; // supplies nutient
			}
			else if (CellsTypes[i] == 0 && i < TumorCellsPair[i]) // tumor cell
			{
				NutrCells[i] = 0;
				Nutrient[i][0] = (Cells[i][0] + Cells[TumorCellsPair[i]][0]) / 2.; // x
				Nutrient[i][1] = (Cells[i][1] + Cells[TumorCellsPair[i]][1]) / 2.; // x
				Nutrient[i][2] = 1.; // level
				Nutrient[i][3] = 1.; // initially consumes nutrient at max level
			}
			else
				NutrCells[i] = -1;
		}
	}

	if (OutputFlg == 1)
		Output();
	VolumeOutput();

	while (t < T - 0.000000000000000001)
	{
		t = t + tau;

		// calculate nutrient level
		if (layers > 0 && Nutr == 1)  // if normal cells exist
		{
			for (int i = 2; i <= N_Cells - 1; i++) // define grid points
			{
				if (CellsTypes[i] == 1) // normal cell
				{
					//NutrCells[i] = 1; // maintained
					Nutrient[i][0] = Cells[i][0]; // x
					Nutrient[i][1] = Cells[i][1]; // y
					//	Nutrient[i][2] = 1.; // level (better maintain from previous step)
				}
				else if (CellsTypes[i] == 0 && i < TumorCellsPair[i]) // tumor cell
				{
					NutrCells[i] = 0;
					Nutrient[i][0] = (Cells[i][0] + Cells[TumorCellsPair[i]][0]) / 2.; // x
					Nutrient[i][1] = (Cells[i][1] + Cells[TumorCellsPair[i]][1]) / 2.; // y
				}
				else
					NutrCells[i] = -1;
			}

			for (int i = 2; i <= N_Cells - 1; i++) // to prevent numerical problems
				if (NutrCells[i] >= 0)
				{
					NutrientBu[i][0] = Nutrient[i][0];
					NutrientBu[i][1] = Nutrient[i][1];
					NutrientBu[i][2] = Nutrient[i][2];
				}

			Achtung = 0;

			NCalcNutr = max(1, NCalcNutr / 2);

			lll = 0;
			while (lll < NCalcNutr)
			{
				for (int i = 2; i <= N_Cells - 1; i++)
				{
					if (NutrCells[i] == 1) // this grid point is normal
						Nutrient[i][2] = Nutrient[i][2] + (tau / NCalcNutr)*Cells[i][2] * Cells[i][2] * (p*Nutrient[i][3] * (1. - Nutrient[i][2]) - (Cells[i][3] * qN)*Nutrient[i][2] / (Nutrient[i][2] + nst));
					else if (NutrCells[i] == 0) // this grid point is tumor
						Nutrient[i][2] = Nutrient[i][2] - (tau / NCalcNutr)*(2.* Cells[i][3] * Cells[i][2] * Cells[i][2] * (qT * Nutrient[i][3] + qN)) * Nutrient[i][2] / (Nutrient[i][2] + nst);
				}

				for (int i = 2; i <= N_Cells - 1; i++)
				{
					if (NutrCells[i] >= 0)
					{
						l = 0;
						while (CellsConnectedWith[i][l][0] != -1)
						{
							CellA = CellsConnectedWith[i][l][0];
							if (CellA > i && (CellsTypes[CellA] == 1 || (CellsTypes[CellA] == 0 && CellA < TumorCellsPair[CellA])))
							{
								dx = max(dR, sqrt((Nutrient[i][0] - Nutrient[CellA][0])*(Nutrient[i][0] - Nutrient[CellA][0]) + (Nutrient[i][1] - Nutrient[CellA][1])*(Nutrient[i][1] - Nutrient[CellA][1])));
								Jg = (tau / NCalcNutr)*Dn*(Nutrient[i][2] - Nutrient[CellA][2]) / dx / dx;
								Nutrient[i][2] = Nutrient[i][2] - Jg;
								Nutrient[CellA][2] = Nutrient[CellA][2] + Jg;
							}
							l++;
						}

						l = 0;
						while (CellsPotConnected[i][l] != -1)
						{
							CellA = CellsPotConnected[i][l];
							if (CellA > i && (CellsTypes[CellA] == 1 || (CellsTypes[CellA] == 0 && CellA < TumorCellsPair[CellA])))
							{
								dx = max(dR, sqrt((Nutrient[i][0] - Nutrient[CellA][0])*(Nutrient[i][0] - Nutrient[CellA][0]) + (Nutrient[i][1] - Nutrient[CellA][1])*(Nutrient[i][1] - Nutrient[CellA][1])));
								Jg = (tau / NCalcNutr)*Dn*(Nutrient[i][2] - Nutrient[CellA][2]) / dx / dx;
								Nutrient[i][2] = Nutrient[i][2] - Jg;
								Nutrient[CellA][2] = Nutrient[CellA][2] + Jg;
							}
							l++;
						}
					}
				}

				for (int i = 2; i <= N_Cells - 1; i++) // to prevent numerical problems
					if (NutrCells[i] >= 0 && (Nutrient[i][2]<0. || Nutrient[i][2]>1.))
						Achtung = 1;

				if (Achtung == 1)
				{
					Achtung = 0;
					lll = -1;
					for (int i = 2; i <= N_Cells - 1; i++)
						if (NutrCells[i] >= 0)
						{
							Nutrient[i][0] = NutrientBu[i][0];
							Nutrient[i][1] = NutrientBu[i][1];
							Nutrient[i][2] = NutrientBu[i][2];
						}
					NCalcNutr = 2 * NCalcNutr;
				}

				lll++;
			}

			for (int i = 2; i <= N_Cells - 1; i++)
				if (NutrCells[i] == 0)
					Nutrient[TumorCellsPair[i]][2] = Nutrient[i][2]; // to ease further calculations

			if (outputEveryAction == 1)
			{
				cout << "\n NUTRIENT LEVEL CALCULATED \n";
				Output();
				system("PAUSE");
			}
		}

		// tumor capsule grows
		if (tumor_capsule == 1)
		{
			Force = 0.;

			l = 0;
			while (CapsulesConnectedWith[1][l][0] != -1)
			{
				BondA = CapsulesConnectedWith[1][l][1];
				CellA = BondsCells[BondA][0];
				CellB = BondsCells[BondA][1];

				if (CellA == 1)
					if (CellsTypes[CellB] == 0) // tumor cells push (or pull)
						Force = Force + min(Smax, Stress(Bonds[BondA][0], kC)) * (Cells[CellB][2] / r0) * Cells[CellB][3] * Bonds[BondA][5];
					else if (CellsTypes[CellB] == 1) // normal cells push
						Force = Force - min(Smax, Stress(Bonds[BondA][0], kC)) * (Cells[CellB][2] / r0);
					else;
				else if (CellB == 1)
					if (CellsTypes[CellA] == 0) // tumor cells push
						Force = Force + min(Smax, Stress(Bonds[BondA][0], kC)) * (Cells[CellA][2] / r0) * Cells[CellA][3] * Bonds[BondA][5];
					else if (CellsTypes[CellA] == 1) // normal cells push
						Force = Force - min(Smax, Stress(Bonds[BondA][0], kC)) * (Cells[CellA][2] / r0);
					else;

					l++;
			}

			Cells[1][2] = Cells[1][2] + tau*(KCT* Force / (2.*PI*Cells[1][2]));

			if (outputEveryAction == 1)
			{
				cout << "\n TUMOR CAPSULE GREW \n";
				Output();
				system("PAUSE");
			}
		}

		// normal capsule grows
		if (norm_capsule == 1)
		{
			Force = 0.;

			l = 0;
			while (CapsulesConnectedWith[0][l][0] != -1)
			{
				BondA = CapsulesConnectedWith[0][l][1];
				CellA = BondsCells[BondA][0];
				CellB = BondsCells[BondA][1];

				if (CellA == 0)
				{
					if (CellsTypes[CellB] == 0) // tumor cell
						Force = Force + min(Smax, Stress(Bonds[BondA][0], kC)) * (Cells[CellB][2] / r0) * Cells[CellB][3] * Bonds[BondA][5];
					else
						Force = Force + min(Smax, Stress(Bonds[BondA][0], kC)) * (Cells[CellB][2] / r0) * Cells[CellB][3] * Bonds[BondA][5];
				}
				else if (CellB == 0)
				{
					if (CellsTypes[CellA] == 0) // tumor cell
						Force = Force + min(Smax, Stress(Bonds[BondA][0], kC)) * (Cells[CellA][2] / r0) * Cells[CellA][3] * Bonds[BondA][5];
					else
						Force = Force + min(Smax, Stress(Bonds[BondA][0], kC)) * (Cells[CellA][2] / r0) * Cells[CellA][3] * Bonds[BondA][5];
				}

				l++;
			}

			Cells[0][2] = Cells[0][2] + tau*KCN* (Force / (2.*PI*Cells[0][2])-s0N);
			
			if (outputEveryAction == 1)
			{
				cout << "\n NORMAL CAPSULE GREW \n";
				Output();
				system("PAUSE");
			}
		}

		// CELLS MOVE 
		kM = sqrt(1. - M*tau);// need to redefine, since tau can be adjusted
		kMn = sqrt(1. - Mn*tau);

		for (int i = 2; i <= N_Cells - 1; i++)
		{
			if (CellsTypes[i] == 1) // normal cell
			{
				// let's define what happens with these cells
				// they shrink until r=rmin, then die if nutrient is low and they are not connected to capsule (including via other cells)
				if (Nutrient[i][2] < nd && Nutrient[i][3] == 0.)
					CellMoveMode[i] = 3;
				// they do nothing active otherwise
				else
					CellMoveMode[i] = 1;

				if (CellMoveMode[i] == 3)
					if (Cells[i][2] > rmin)
						Cells[i][2] = kMn*Cells[i][2];
					else
						CellKill(i);

				// force projections
				F[0] = 0.;
				F[1] = 0.;

				// bonds
				l = 0;
				while (CellsConnectedWith[i][l][0] != -1)
				{
					BondA = CellsConnectedWith[i][l][1];
					CellA = BondsCells[BondA][0];
					CellB = BondsCells[BondA][1];

					if (BondsCells[BondA][2] >= 1 && BondsCells[BondA][2] <= 4)
						ltest2 = sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1])) - Cells[CellA][2] - Cells[CellB][2];
					else
						ltest2 = Bonds[BondA][0];

					ltest = (ltest2 / abs(ltest2))*sqrt((Bonds[BondA][1] - Bonds[BondA][3])*(Bonds[BondA][1] - Bonds[BondA][3]) + (Bonds[BondA][2] - Bonds[BondA][4])*(Bonds[BondA][2] - Bonds[BondA][4]));

					if (CellsTypes[CellA] + CellsTypes[CellB] == 1) // connected with tumor cell
						Force = min(Smax, Stress(Bonds[BondA][0], kTN))* Bonds[BondA][5] * Cells[CellB][3] * min(Cells[CellA][2], Cells[CellB][2]) / r0;
					else // connected with normal cell or capsule
						Force = min(Smax, Stress(Bonds[BondA][0], kN))* Bonds[BondA][5] * Cells[CellB][3] * min(Cells[CellA][2], Cells[CellB][2]) / r0;

					if (CellA == i)
					{
						F[0] = F[0] + Force*(Bonds[BondA][1] - Bonds[BondA][3]) / ltest;
						F[1] = F[1] + Force*(Bonds[BondA][2] - Bonds[BondA][4]) / ltest;
					}

					if (CellB == i)
					{
						F[0] = F[0] + Force*(Bonds[BondA][3] - Bonds[BondA][1]) / ltest;
						F[1] = F[1] + Force*(Bonds[BondA][4] - Bonds[BondA][2]) / ltest;
					}

					l++;
				}
				// so it moves in x direction
				Cells[i][0] = Cells[i][0] + K*F[0] * tau / (PI*Cells[i][2] * Cells[i][2]);
				// and in y direction
				Cells[i][1] = Cells[i][1] + K*F[1] * tau / (PI*Cells[i][2] * Cells[i][2]);
			}
			// else -- tumor cells
			else if (CellsTypes[i] == 0 && TumorCellsPair[i] > i) // to not duplicate
			{
				// dividing tumor cells are considered together
				CellA = i;
				CellB = TumorCellsPair[i];
				D = sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1]));

				JoinX = (Cells[CellB][0] - Cells[CellA][0]) / D;
				JoinY = (Cells[CellB][1] - Cells[CellA][1]) / D;
				CenterX = (Cells[CellA][0] + Cells[CellB][0]) / 2.;
				CenterY = (Cells[CellA][1] + Cells[CellB][1]) / 2.;

				// force projections
				F[0] = 0.;
				F[1] = 0.;

				Torque = 0.;

				PressureOnCell[CellA] = 0.;
				PressureOnCell[CellB] = 0.;

				// let's define what happens with these cells
				// if normal cells exists and nutrient is accounted for then
				if (layers > 0 && Nutr == 1)
					// cells move apart in the process of division if nutrient is enough and their radii are r0
					if (Cells[CellA][2] == r0 && Nutrient[CellA][2] > nq)
						CellMoveMode[i] = 1;
				// their radii grow if nutrient is enough and their radii are less than r0
					else if (Cells[CellA][2] < r0 && Nutrient[CellA][2] > nq)
						CellMoveMode[i] = 2;
				// they they shrink until r=rmin, then die if nutrient is low
					else if (Nutrient[CellA][2] < nd)
						CellMoveMode[i] = 3;
					else
						CellMoveMode[i] = 0; // they are quiescent
				else
					// otherwise all tumor cells constantly proliferate
					CellMoveMode[i] = 1;

				for (int ij = 0; ij <= 1; ij++)
				{
					if (ij == 0)
						CellE = CellA;
					else
						CellE = CellB;

					l = 0;
					while (CellsConnectedWith[CellE][l][0] != -1)
					{
						BondA = CellsConnectedWith[CellE][l][1];

						if (BondsCells[BondA][2] >= 1 && BondsCells[BondA][2] <= 8)
						{
							CellC = BondsCells[BondA][0];
							CellD = BondsCells[BondA][1];

							kF = Bonds[BondA][5] * Cells[CellC][3] * Cells[CellD][3] * min(Cells[CellC][2], Cells[CellD][2]) / r0;

							if (CellsTypes[CellC] + CellsTypes[CellD] == 1) // connected with normal cell
								ForcePr = kF*Stress(Bonds[BondA][0], kTN);
							else // connected with tumor cell or capsule
								ForcePr = kF*Stress(Bonds[BondA][0], kT);

							Force = min(kF*Smax, ForcePr);

							if (BondsCells[BondA][2] >= 1 && BondsCells[BondA][2] <= 4)
								ltest2 = sqrt((Cells[CellC][0] - Cells[CellD][0])*(Cells[CellC][0] - Cells[CellD][0]) + (Cells[CellC][1] - Cells[CellD][1])*(Cells[CellC][1] - Cells[CellD][1])) - Cells[CellC][2] - Cells[CellD][2];
							else
								ltest2 = Bonds[BondA][0];

							ltest = (ltest2 / abs(ltest2))*sqrt((Bonds[BondA][1] - Bonds[BondA][3])*(Bonds[BondA][1] - Bonds[BondA][3]) + (Bonds[BondA][2] - Bonds[BondA][4])*(Bonds[BondA][2] - Bonds[BondA][4]));
						
							if (CellC == CellE) // our cell is the first one in the bond
							{
								begx = Bonds[BondA][1];
								begy = Bonds[BondA][2];
								endx = Bonds[BondA][3];
								endy = Bonds[BondA][4];
							}
							else // our cell is the second one in the bond
							{
								begx = Bonds[BondA][3];
								begy = Bonds[BondA][4];
								endx = Bonds[BondA][1];
								endy = Bonds[BondA][2];
							}

							ForceX = Force*(begx - endx) / ltest;
							ForceY = Force*(begy - endy) / ltest;

							ForcePrX = ForcePr*(begx - endx) / ltest;
							ForcePrY = ForcePr*(begy - endy) / ltest;

							F[0] = F[0] + ForceX;
							F[1] = F[1] + ForceY;

							Torque = Torque + (begx - CenterX)*ForceY - (begy - CenterY)*ForceX;

							if (CellMoveMode[i] == 1) // we need projection of forces on the division line
							{
								if (BondsCells[BondA][2] == 1 || BondsCells[BondA][2] == 5 || BondsCells[BondA][2] == 7 || (CellC == CellE && BondsCells[BondA][2] == 2) || (CellC != CellE && BondsCells[BondA][2] == 3))
									PressureOnCell[CellE] = PressureOnCell[CellE] + (1. - 2.*ij)*(ForcePrX*JoinX + ForcePrY*JoinY);
								else //if (BondsCells[BondA][2] == 4) // double closed bond
									PressureOnCell[TumorCellsPair[CellE]] = PressureOnCell[TumorCellsPair[CellE]] - (1. - 2.*ij)*(ForcePrX*JoinX + ForcePrY*JoinY);
							}
							if (CellMoveMode[i] == 2) // we need projection of forces on the line towards the center of mass of dividing cells, and it sums for two cells
								PressureOnCell[CellA] = PressureOnCell[CellA] + (ForcePrX*(CenterX - begx) + ForcePrY*(CenterY - begy)) / sqrt((CenterX - begx)*(CenterX - begx) + (CenterY - begy)*(CenterY - begy));
						}
						l++;
					}
				}

				// cells move apart in the process of division
				if (CellMoveMode[i] == 1)
				{
					dRapartA = -1.*v* tau*(Scr - max(min(PressureOnCell[CellA], Scr), 0.)) / Scr;
					dXapartA = dRapartA*JoinX;
					dYapartA = dRapartA*JoinY;

					dRapartB = v* tau*(Scr - max(min(PressureOnCell[CellB], Scr), 0.)) / Scr;
					dXapartB = dRapartB*JoinX;
					dYapartB = dRapartB*JoinY;

					Nutrient[i][3] = ((Scr - max(min(PressureOnCell[CellA], Scr), 0.)) + (Scr - max(min(PressureOnCell[CellB], Scr), 0.))) / Scr / 2.;

					Cells[CellA][0] = Cells[CellA][0] + dXapartA;
					Cells[CellA][1] = Cells[CellA][1] + dYapartA;
					Cells[CellB][0] = Cells[CellB][0] + dXapartB;
					Cells[CellB][1] = Cells[CellB][1] + dYapartB;

					// after this their fractional areas need to be redefined
					D = sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1]));
					if (D / (2.*Cells[CellA][2]) > 1.)
						alpha = 0.;
					else
						alpha = 2 * acos(D / (2.*Cells[CellA][2]));
					fS = (2 * PI - alpha + sin(alpha)) / (2 * PI);

					Cells[CellA][3] = fS;
					Cells[CellB][3] = fS;
				}
				// or they grow
				else if (CellMoveMode[i] == 2)
				{
					ScrGrow = 2.*Cells[CellA][3] * (Cells[CellA][2] / r0)*Scr;

					Nutrient[i][3] = (ScrGrow - max(min(PressureOnCell[CellA], ScrGrow), 0.)) / ScrGrow;

					kB = sqrt(1. + B* Nutrient[i][3] * tau);
					Cells[CellA][2] = min(r0, kB*Cells[CellA][2]);
					kb = Cells[CellA][2] / Cells[CellB][2];
					Cells[CellB][2] = Cells[CellA][2];

					Cells[CellA][0] = kb*(Cells[CellA][0] - CenterX) + CenterX;
					Cells[CellA][1] = kb*(Cells[CellA][1] - CenterY) + CenterY;
					Cells[CellB][0] = kb*(Cells[CellB][0] - CenterX) + CenterX;
					Cells[CellB][1] = kb*(Cells[CellB][1] - CenterY) + CenterY;

					D = sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1])); // will be needed
				}
				// or they are in dying zone
				else if (CellMoveMode[i] == 3)
				{
					Nutrient[i][3] = 0.;

					if (Cells[CellA][2] > rmin)
					{
						Cells[CellA][2] = kM*Cells[CellA][2];
						Cells[CellB][2] = Cells[CellA][2];

						Cells[CellA][0] = kM*(Cells[CellA][0] - CenterX) + CenterX;
						Cells[CellA][1] = kM*(Cells[CellA][1] - CenterY) + CenterY;
						Cells[CellB][0] = kM*(Cells[CellB][0] - CenterX) + CenterX;
						Cells[CellB][1] = kM*(Cells[CellB][1] - CenterY) + CenterY;
					}
					else
					{
						CellKill(CellA);
						CellKill(CellB);
					}

					D = sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1])); // will be needed
				}
				else // CellMoveMode = 0, quiescent
					Nutrient[i][3] = 0.;

				Nutrient[CellB][3] = Nutrient[i][3];

				// in any case they rotate wrt center mass due to external forces

				if (D < 2.*Cells[CellA][2])
					I = 0.25*(Cells[CellA][2] * Cells[CellB][2])* (2.*(D*D + 2.*Cells[CellA][2] * Cells[CellB][2])*(PI - acos(D / (2.*Cells[CellA][2]))) + 3.*D*sqrt(4.*Cells[CellA][2] * Cells[CellB][2] - D*D)); // always Cells[CellA][0] = Cells[CellB][0]
				else
					I = 3.* PI*Cells[CellA][2] * Cells[CellB][2]*Cells[CellA][2] * Cells[CellB][2]; // always Cells[CellA][0] = Cells[CellB][0]

				dPhi = K*Torque*tau / I;

				Cells[CellA][0] = (Cells[CellA][0] - CenterX)*cos(dPhi) - (Cells[CellA][1] - CenterY)*sin(dPhi) + CenterX;
				Cells[CellA][1] = (Cells[CellA][0] - CenterX)*sin(dPhi) + (Cells[CellA][1] - CenterY)*cos(dPhi) + CenterY;

				Cells[CellB][0] = (Cells[CellB][0] - CenterX)*cos(dPhi) - (Cells[CellB][1] - CenterY)*sin(dPhi) + CenterX;
				Cells[CellB][1] = (Cells[CellB][0] - CenterX)*sin(dPhi) + (Cells[CellB][1] - CenterY)*cos(dPhi) + CenterY;

				// due to numerical errors, for large dPhi correction is needed, otherwise dividing cells may get closer with time

				krot = D / sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1]));

				Cells[CellA][0] = CenterX + krot*(Cells[CellA][0] - CenterX);
				Cells[CellB][0] = CenterX + krot*(Cells[CellB][0] - CenterX);
				Cells[CellA][1] = CenterY + krot*(Cells[CellA][1] - CenterY);
				Cells[CellB][1] = CenterY + krot*(Cells[CellB][1] - CenterY);

				// and they both move due to external forces

				dXboth = K*F[0] * tau / (2.*PI* Cells[CellA][3] * Cells[CellA][2] * Cells[CellA][2]);
				dYboth = K*F[1] * tau / (2.*PI* Cells[CellA][3] * Cells[CellA][2] * Cells[CellA][2]);

				Cells[CellA][0] = Cells[CellA][0] + dXboth;
				Cells[CellA][1] = Cells[CellA][1] + dYboth;
				Cells[CellB][0] = Cells[CellB][0] + dXboth;
				Cells[CellB][1] = Cells[CellB][1] + dYboth;

				// tumor cells cannot go through the capsules
				/*	for (int ij = 1 - min(1, norm_capsule); ij <= tumor_capsule; ij++)
				{
				if (sqrt(Cells[CellA][0] * Cells[CellA][0] + Cells[CellA][1] * Cells[CellA][1]) + Cells[CellA][2] > Cells[ij][2])
				{
				dXboth = 0.999*Cells[CellA][0] * (Cells[ij][2] - Cells[CellA][2]) / (sqrt(Cells[CellA][0] * Cells[CellA][0] + Cells[CellA][1] * Cells[CellA][1])) - Cells[CellA][0];
				dYboth = 0.999*Cells[CellA][1] * (Cells[ij][2] - Cells[CellA][2]) / (sqrt(Cells[CellA][0] * Cells[CellA][0] + Cells[CellA][1] * Cells[CellA][1])) - Cells[CellA][1];

				Cells[CellA][0] = Cells[CellA][0] + dXboth;
				Cells[CellB][0] = Cells[CellB][0] + dXboth;
				Cells[CellA][1] = Cells[CellA][1] + dYboth;
				Cells[CellB][1] = Cells[CellB][1] + dYboth;
				}

				if (sqrt(Cells[CellB][0] * Cells[CellB][0] + Cells[CellB][1] * Cells[CellB][1]) + Cells[CellB][2] > Cells[ij][2])
				{
				dXboth = 0.999*Cells[CellB][0] * (Cells[ij][2] - Cells[CellB][2]) / (sqrt(Cells[CellB][0] * Cells[CellB][0] + Cells[CellB][1] * Cells[CellB][1])) - Cells[CellB][0];
				dYboth = 0.999*Cells[CellB][1] * (Cells[ij][2] - Cells[CellB][2]) / (sqrt(Cells[CellB][0] * Cells[CellB][0] + Cells[CellB][1] * Cells[CellB][1])) - Cells[CellB][1];

				Cells[CellA][0] = Cells[CellA][0] + dXboth;
				Cells[CellB][0] = Cells[CellB][0] + dXboth;
				Cells[CellA][1] = Cells[CellA][1] + dYboth;
				Cells[CellB][1] = Cells[CellB][1] + dYboth;
				}
				}*/
			}

			// normal cells cannot go through the normal capsule
			/*if (norm_capsule > 0)
			{
			l = 0;
			while (CapsulesConnectedWith[0][l][0] != -1)
			{
			CellAA = CapsulesConnectedWith[0][l][0];
			if (sqrt(Cells[CellAA][0] * Cells[CellAA][0] + Cells[CellAA][1] * Cells[CellAA][1]) + Cells[CellAA][2] > Cells[0][2])
			{
			Cells[CellAA][0] = 0.999*Cells[CellAA][0] * (Cells[0][2] - Cells[CellAA][2]) / (sqrt(Cells[CellAA][0] * Cells[CellAA][0] + Cells[CellAA][1] * Cells[CellAA][1]));
			Cells[CellAA][1] = 0.999*Cells[CellAA][1] * (Cells[0][2] - Cells[CellAA][2]) / (sqrt(Cells[CellAA][0] * Cells[CellAA][0] + Cells[CellAA][1] * Cells[CellAA][1]));
			}
			l++;
			}
			}	*/
		}

		if (outputEveryAction == 1)
		{
			cout << "\n CELLS MOVED \n";
			Output();
			system("PAUSE");
		}

		// After cells have moved and changed sizes, bonds become redefined and break if long
		WorstShrink = 1.;
		for (int i = 1; i <= N_Bonds - 1; i++)
			if (BondsCells[i][2] >= 1 && BondsCells[i][2] <= 4)
			{
				Lprev = (Bonds[i][0] / abs(Bonds[i][0]))*sqrt((Bonds[i][3] - Bonds[i][1])*(Bonds[i][3] - Bonds[i][1]) + (Bonds[i][4] - Bonds[i][2])*(Bonds[i][4] - Bonds[i][2])) + r0;

				CellA = BondsCells[i][0];
				CellB = BondsCells[i][1];

				// length is this, if there are no hidden parts -- in this case will be rewritten below
				ltest = sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1])) - Cells[CellA][2] - Cells[CellB][2];
				Bonds[i][0] = ltest; // positions are these, if not broken -- in this case will be rewritten below
				Bonds[i][1] = Cells[CellA][0] + (Cells[CellB][0] - Cells[CellA][0])*Cells[CellA][2] / (Cells[CellA][2] + Cells[CellB][2] + ltest);
				Bonds[i][2] = Cells[CellA][1] + (Cells[CellB][1] - Cells[CellA][1])*Cells[CellA][2] / (Cells[CellA][2] + Cells[CellB][2] + ltest);
				Bonds[i][3] = Cells[CellB][0] + (Cells[CellA][0] - Cells[CellB][0])*Cells[CellB][2] / (Cells[CellB][2] + Cells[CellA][2] + ltest);
				Bonds[i][4] = Cells[CellB][1] + (Cells[CellA][1] - Cells[CellB][1])*Cells[CellB][2] / (Cells[CellB][2] + Cells[CellA][2] + ltest);
				Bonds[i][5] = 1.;

				BondsCells[i][2] = 1; // yet

									  // if some part of bond is hidden in tumor-dividing-brother-cell, we need to account for it
				if (ltest > 0.) // but only if bond length is positive -- otherwise cells should repel fast
					for (int iii = 0; iii <= 1; iii++)
					{
						CellD = BondsCells[i][iii]; // CellA or CellB

						if (CellsTypes[CellD] == 0)
						{
							CellC = TumorCellsPair[CellD];

							if (iii == 0)
							{
								begx = Bonds[i][1];
								begy = Bonds[i][2];
								endx = Bonds[i][3];
								endy = Bonds[i][4];
							}
							else
							{
								begx = Bonds[i][3];
								begy = Bonds[i][4];
								endx = Bonds[i][1];
								endy = Bonds[i][2];
							}

							L = sqrt((begx - Cells[CellC][0])*(begx - Cells[CellC][0]) + (begy - Cells[CellC][1])*(begy - Cells[CellC][1]));

							alpha = acos(((endx - begx)*(Cells[CellC][0] - begx) + (endy - begy)*(Cells[CellC][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
							if (alpha < PI / 2.)
								L = L*sin(alpha);

							if (L < Cells[CellC][2])
							{
								D = sqrt((Cells[CellD][0] - Cells[CellC][0])*(Cells[CellD][0] - Cells[CellC][0]) + (Cells[CellD][1] - Cells[CellC][1])*(Cells[CellD][1] - Cells[CellC][1]));

								if (iii == 0)
									cosbeta = ((Bonds[i][3] - Bonds[i][1])*(Cells[CellC][0] - Cells[CellD][0]) + (Bonds[i][4] - Bonds[i][2])*(Cells[CellC][1] - Cells[CellD][1])) / (ltest *D);
								else
									cosbeta = ((Bonds[i][1] - Bonds[i][3])*(Cells[CellC][0] - Cells[CellD][0]) + (Bonds[i][2] - Bonds[i][4])*(Cells[CellC][1] - Cells[CellD][1])) / (ltest *D);

								if (D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2] > 0.) // just in case, numerical errors are possible
									LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2]) - Cells[CellD][2]; // "hidden" length of bond
								else
									LH = 0.;

								if (LH > 0.) // just in case, numerical errors are possible
								{
									Bonds[i][0] = Bonds[i][0] - LH; // effective length 
									Bonds[i][5] = Bonds[i][5] * (1. + ((1. - 2.*Cells[CellD][3]) / Cells[CellD][3]) *(LH / D));
									if (BondsCells[i][2] == 1)
										BondsCells[i][2] = 3 - iii;
									else if (BondsCells[i][2] > 1)
										BondsCells[i][2] = 4;
								}
							}
						}
					}

				if (Bonds[i][0] > lmax) // length of "open" part of bond
				{
					// correct connection matrix
					if (CellA == 1)//tumor caspule
					{
						l = 0;
						while (CapsulesConnectedWith[CellA][l][0] != CellB)
							l++;
						while (CapsulesConnectedWith[CellA][l][0] != -1)
						{
							CapsulesConnectedWith[CellA][l][0] = CapsulesConnectedWith[CellA][l + 1][0];
							CapsulesConnectedWith[CellA][l][1] = CapsulesConnectedWith[CellA][l + 1][1];
							l++;
						}
					}
					else
					{
						l = 0;
						while (CellsConnectedWith[CellA][l][0] != CellB)
							l++;
						while (CellsConnectedWith[CellA][l][0] != -1)
						{
							CellsConnectedWith[CellA][l][0] = CellsConnectedWith[CellA][l + 1][0];
							CellsConnectedWith[CellA][l][1] = CellsConnectedWith[CellA][l + 1][1];
							l++;
						}
					}

					if (CellB == 1)//tumor caspule
					{
						l = 0;
						while (CapsulesConnectedWith[CellB][l][0] != CellA)
							l++;
						while (CapsulesConnectedWith[CellB][l][0] != -1)
						{
							CapsulesConnectedWith[CellB][l][0] = CapsulesConnectedWith[CellB][l + 1][0];
							CapsulesConnectedWith[CellB][l][1] = CapsulesConnectedWith[CellB][l + 1][1];
							l++;
						}
					}
					else
					{
						l = 0;
						while (CellsConnectedWith[CellB][l][0] != CellA)
							l++;
						while (CellsConnectedWith[CellB][l][0] != -1)
						{
							CellsConnectedWith[CellB][l][0] = CellsConnectedWith[CellB][l + 1][0];
							CellsConnectedWith[CellB][l][1] = CellsConnectedWith[CellB][l + 1][1];
							l++;
						}
					}

					Bonds[i][0] = 0.;
					Bonds[i][1] = 0.;
					Bonds[i][2] = 0.;
					Bonds[i][3] = 0.;
					Bonds[i][4] = 0.;
					Bonds[i][5] = 1.;

					BondsCells[i][0] = 0;
					BondsCells[i][1] = 0;
					BondsCells[i][2] = 0;
				}
				else // bond not broken
				{
					if (Lprev > 0.)
						WorstShrink = min(WorstShrink, ((Bonds[i][0] / abs(Bonds[i][0]))*sqrt((Bonds[i][3] - Bonds[i][1])*(Bonds[i][3] - Bonds[i][1]) + (Bonds[i][4] - Bonds[i][2])*(Bonds[i][4] - Bonds[i][2])) + r0) / Lprev);
				}
			}
			else if (BondsCells[i][2] >= 5 && BondsCells[i][2] <= 8) // bond with normal capsule or (tumor cells with) tumor capsule
			{
				Lprev = (Bonds[i][0] / abs(Bonds[i][0]))*sqrt((Bonds[i][3] - Bonds[i][1])*(Bonds[i][3] - Bonds[i][1]) + (Bonds[i][4] - Bonds[i][2])*(Bonds[i][4] - Bonds[i][2])) + r0;

				CellA = BondsCells[i][0];
				CellB = BondsCells[i][1]; // one of capsules, 0 or 1

				ltest = Cells[CellB][2] - sqrt((Cells[CellA][0])*(Cells[CellA][0]) + (Cells[CellA][1])*(Cells[CellA][1])) - Cells[CellA][2];
				Bonds[i][0] = ltest;
				Bonds[i][1] = Cells[CellA][0] + Cells[CellA][2] * Cells[CellA][0] / sqrt(Cells[CellA][0] * Cells[CellA][0] + Cells[CellA][1] * Cells[CellA][1]);
				Bonds[i][2] = Cells[CellA][1] + Cells[CellA][2] * Cells[CellA][1] / sqrt(Cells[CellA][0] * Cells[CellA][0] + Cells[CellA][1] * Cells[CellA][1]);
				Bonds[i][3] = Cells[CellA][0] + (Cells[CellA][2] + ltest) * Cells[CellA][0] / sqrt(Cells[CellA][0] * Cells[CellA][0] + Cells[CellA][1] * Cells[CellA][1]);
				Bonds[i][4] = Cells[CellA][1] + (Cells[CellA][2] + ltest) * Cells[CellA][1] / sqrt(Cells[CellA][0] * Cells[CellA][0] + Cells[CellA][1] * Cells[CellA][1]);

				BondsCells[i][2] = (BondsCells[i][2] + 1) / 2 + (BondsCells[i][2] - 1) / 2; // yet open

				Bonds[i][5] = 1.;

				// if some part of bond is hidden in tumor-dividing-brother-cell, we need to account for it
				if (ltest > 0.)
					if (CellsTypes[CellA] == 0)
					{
						CellC = TumorCellsPair[CellA];
						begx = Bonds[i][1];
						begy = Bonds[i][2];
						endx = Bonds[i][3];
						endy = Bonds[i][4];

						L = sqrt((begx - Cells[CellC][0])*(begx - Cells[CellC][0]) + (begy - Cells[CellC][1])*(begy - Cells[CellC][1]));

						alpha = acos(((endx - begx)*(Cells[CellC][0] - begx) + (endy - begy)*(Cells[CellC][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
						if (alpha < PI / 2.)
							L = L*sin(alpha);

						if (L < Cells[CellC][2])
						{
							D = sqrt((Cells[CellA][0] - Cells[CellC][0])*(Cells[CellA][0] - Cells[CellC][0]) + (Cells[CellA][1] - Cells[CellC][1])*(Cells[CellA][1] - Cells[CellC][1]));

							cosbeta = ((endx - begx)*(Cells[CellC][0] - Cells[CellA][0]) + (endy - begy)*(Cells[CellC][1] - Cells[CellA][1])) / (ltest * D);

							if (D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2] > 0.) // just in case, numerical errors are possible
								LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2]) - Cells[CellA][2]; // "hidden" length of bond
							else
								LH = 0.;

							if (LH > 0.) // just in case, numerical errors are possible
							{
								Bonds[i][0] = Bonds[i][0] - LH; // effective length 
								Bonds[i][5] = (1. + ((1. - 2.*Cells[CellA][3]) / Cells[CellA][3]) *(LH / D));
								BondsCells[i][2]++;
							}
						}
					}
				// now we know Bonds[i][0] in all cases

				if (Bonds[i][0] > lmax)
				{
					// correct connection matrix
					if (CellA <= 1) // capsule
					{
						l = 0;
						while (CapsulesConnectedWith[CellA][l][0] != CellB)
							l++;
						while (CapsulesConnectedWith[CellA][l][0] != -1)
						{
							CapsulesConnectedWith[CellA][l][0] = CapsulesConnectedWith[CellA][l + 1][0];
							CapsulesConnectedWith[CellA][l][1] = CapsulesConnectedWith[CellA][l + 1][1];
							l++;
						}
					}
					else
					{
						l = 0;
						while (CellsConnectedWith[CellA][l][0] != CellB)
							l++;
						while (CellsConnectedWith[CellA][l][0] != -1)
						{
							CellsConnectedWith[CellA][l][0] = CellsConnectedWith[CellA][l + 1][0];
							CellsConnectedWith[CellA][l][1] = CellsConnectedWith[CellA][l + 1][1];
							l++;
						}
					}

					if (CellB <= 1) // capsule
					{
						l = 0;
						while (CapsulesConnectedWith[CellB][l][0] != CellA)
							l++;
						while (CapsulesConnectedWith[CellB][l][0] != -1)
						{
							CapsulesConnectedWith[CellB][l][0] = CapsulesConnectedWith[CellB][l + 1][0];
							CapsulesConnectedWith[CellB][l][1] = CapsulesConnectedWith[CellB][l + 1][1];
							l++;
						}
					}
					else
					{
						l = 0;
						while (CellsConnectedWith[CellB][l][0] != CellA)
							l++;
						while (CellsConnectedWith[CellB][l][0] != -1)
						{
							CellsConnectedWith[CellB][l][0] = CellsConnectedWith[CellB][l + 1][0];
							CellsConnectedWith[CellB][l][1] = CellsConnectedWith[CellB][l + 1][1];
							l++;
						}
					}

					Bonds[i][0] = 0.;
					Bonds[i][1] = 0.;
					Bonds[i][2] = 0.;
					Bonds[i][3] = 0.;
					Bonds[i][4] = 0.;
					Bonds[i][5] = 1.;

					BondsCells[i][0] = 0;
					BondsCells[i][1] = 0;
					BondsCells[i][2] = 0;
				}
				else // bond not broken
				{
					if (Lprev > 0.)
						WorstShrink = min(WorstShrink, ((Bonds[i][0] / abs(Bonds[i][0]))*sqrt((Bonds[i][3] - Bonds[i][1])*(Bonds[i][3] - Bonds[i][1]) + (Bonds[i][4] - Bonds[i][2])*(Bonds[i][4] - Bonds[i][2])) + r0) / Lprev);
				}
			}

		if (outputEveryAction == 1)
		{
			cout << "\n BONDS REDEFINED \n";
			Output();
			system("PAUSE");
		}

		// cells divide
		N_BondsOld2 = N_Bonds;
		for (int i = 1; i <= N_BondsOld2 - 1; i++)
			if (BondsCells[i][2] == 9) // bond between dividing cells
			{
				CellA = BondsCells[i][0];
				CellB = BondsCells[i][1];

				D = sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1]));

				JoinX = (Cells[CellB][0] - Cells[CellA][0]) / D;
				JoinY = (Cells[CellB][1] - Cells[CellA][1]) / D;

				Bonds[i][0] = D - Cells[CellA][2] - Cells[CellB][2];
				Bonds[i][1] = Cells[CellA][0] + Cells[CellA][2] * JoinX;
				Bonds[i][2] = Cells[CellA][1] + Cells[CellA][2] * JoinY;
				Bonds[i][3] = Cells[CellB][0] - Cells[CellB][2] * JoinX;
				Bonds[i][4] = Cells[CellB][1] - Cells[CellB][2] * JoinY;

				if (Bonds[i][0] > 0. || (CellA + CellB == 5 && (tumor_capsule == 1 || layers > 0)))// if division bond becomes positive // tumor_capsule == 1 && 
				{
					//new divisions of two cells happen immediately
					for (int iii = 0; iii <= 1; iii++)
					{
						if (iii == 0)
							CellC = CellA;
						else
							CellC = CellB;

						MaxForce = 0.;
						DivDir = 0.;

						// define direction of division
						l = 0;
						while (CellsConnectedWith[CellC][l][0] != -1)
						{
							BondA = CellsConnectedWith[CellC][l][1];
							if (CellsTypes[CellC] + CellsTypes[CellsConnectedWith[CellC][l][0]] == 1) // connected with normal cell
								Force = Stress(Bonds[BondA][0], kTN)*Bonds[BondA][5] * Cells[BondsCells[BondA][0]][3] * Cells[BondsCells[BondA][1]][3] * min(Cells[BondsCells[BondA][0]][2], Cells[BondsCells[BondA][1]][2]) / r0;
							else
								Force = Stress(Bonds[BondA][0], kT)*Bonds[BondA][5] * Cells[BondsCells[BondA][0]][3] * Cells[BondsCells[BondA][1]][3] * min(Cells[BondsCells[BondA][0]][2], Cells[BondsCells[BondA][1]][2]) / r0;

							if (Force > MaxForce)
							{
								MaxForce = Force;
								DivDir = atan((Bonds[BondA][2] - Bonds[BondA][4]) / (Bonds[BondA][1] - Bonds[BondA][3])) + PI / 2.;
							}
							l++;
						}

						Cells[CellC][0] = Cells[CellC][0] + dR*cos(DivDir); // x
						Cells[CellC][1] = Cells[CellC][1] + dR*sin(DivDir); // y
						Cells[CellC][3] = fSinit;

						Cells[N_Cells][0] = Cells[CellC][0] - dR*cos(DivDir); // x
						Cells[N_Cells][1] = Cells[CellC][1] - dR*sin(DivDir); // y
						Cells[N_Cells][2] = r0; // radius
						Cells[N_Cells][3] = fSinit;
						CellsTypes[N_Cells] = 0;

						Nutrient[N_Cells][2] = Nutrient[CellC][2]; // to ease calculations of nutrient level

						TumorCellsPair[CellC] = N_Cells;
						TumorCellsPair[N_Cells] = CellC;

						CellMoveMode[N_Cells] = CellMoveMode[CellC];

						D = sqrt((Cells[CellC][0] - Cells[N_Cells][0])*(Cells[CellC][0] - Cells[N_Cells][0]) + (Cells[CellC][1] - Cells[N_Cells][1])*(Cells[CellC][1] - Cells[N_Cells][1]));

						JoinX = (Cells[N_Cells][0] - Cells[CellC][0]) / D;
						JoinY = (Cells[N_Cells][1] - Cells[CellC][1]) / D;

						ll = 0;
						while (CellsConnectedWith[CellC][ll][0] != -1)
							ll++;
						if (ll >= MaxBondsForCell - 10)
							IncreaseMaxBondsForCell();
						CellsConnectedWith[CellC][ll][0] = N_Cells;
						CellsConnectedWith[CellC][ll][1] = N_Bonds;

						ll = 0;
						while (CellsConnectedWith[N_Cells][ll][0] != -1)
							ll++;
						CellsConnectedWith[N_Cells][ll][0] = CellC;
						CellsConnectedWith[N_Cells][ll][1] = N_Bonds;

						// dividing bond between these cells
						BondsCells[N_Bonds][0] = CellC;
						BondsCells[N_Bonds][1] = N_Cells;
						BondsCells[N_Bonds][2] = 9;

						Bonds[N_Bonds][0] = -2.*Cells[CellC][2] + 2.*dR;
						Bonds[N_Bonds][1] = Cells[CellC][0] + Cells[CellC][2] * JoinX;
						Bonds[N_Bonds][2] = Cells[CellC][1] + Cells[CellC][2] * JoinY;
						Bonds[N_Bonds][3] = Cells[CellC][0] - Cells[CellC][2] * JoinX;
						Bonds[N_Bonds][4] = Cells[CellC][1] - Cells[CellC][2] * JoinY;

						Bonds[N_Bonds][5] = 1.;

						N_Bonds++;
						if (N_Bonds >= BondsMaxN - 10)
						{
							i = IncreaseBondsMaxN(i);
							N_BondsOld2 = N_Bonds; // this actually includes bonds formed during this cycle, but that's ok
						}

						// we need to bond new daughters with all cells that their mothers were connected to
						// and rebond old bonds since they can become closed and change length a bit
						l = 0;
						while (CellsConnectedWith[CellC][l][0] != -1)
						{
							if (BondsCells[CellsConnectedWith[CellC][l][1]][2] >= 1 && BondsCells[CellsConnectedWith[CellC][l][1]][2] <= 4)
							{
								CellD = CellsConnectedWith[CellC][l][0];

								// CellD cannot be tumor capsule here, so
								ll = 0;
								while (CellsConnectedWith[CellD][ll][0] != -1)
									ll++;
								CellsConnectedWith[CellD][ll][0] = N_Cells;
								CellsConnectedWith[CellD][ll][1] = N_Bonds;

								ll = 0;
								while (CellsConnectedWith[N_Cells][ll][0] != -1)
									ll++;
								CellsConnectedWith[N_Cells][ll][0] = CellD;
								CellsConnectedWith[N_Cells][ll][1] = N_Bonds;

								for (int ij = 0; ij <= 1; ij++)
								{
									if (ij == 0)
									{
										CellAA = CellC;
										BondA = CellsConnectedWith[CellC][l][1];
									}
									else
									{
										CellAA = N_Cells;
										BondA = N_Bonds;
									}

									BondsCells[BondA][0] = CellD;
									BondsCells[BondA][1] = CellAA;
									BondsCells[BondA][2] = 1; // yet

															  // length is this, if there are no hidden parts -- in this case will be rewritten below
									ltest = sqrt((Cells[CellD][0] - Cells[CellAA][0])*(Cells[CellD][0] - Cells[CellAA][0]) + (Cells[CellD][1] - Cells[CellAA][1])*(Cells[CellD][1] - Cells[CellAA][1])) - Cells[CellD][2] - Cells[CellAA][2];
									Bonds[BondA][0] = ltest;
									// positions are these
									Bonds[BondA][1] = Cells[CellD][0] + (Cells[CellAA][0] - Cells[CellD][0])*Cells[CellD][2] / (Cells[CellD][2] + Cells[CellAA][2] + ltest);
									Bonds[BondA][2] = Cells[CellD][1] + (Cells[CellAA][1] - Cells[CellD][1])*Cells[CellD][2] / (Cells[CellD][2] + Cells[CellAA][2] + ltest);
									Bonds[BondA][3] = Cells[CellAA][0] + (Cells[CellD][0] - Cells[CellAA][0])*Cells[CellAA][2] / (Cells[CellAA][2] + Cells[CellD][2] + ltest);
									Bonds[BondA][4] = Cells[CellAA][1] + (Cells[CellD][1] - Cells[CellAA][1])*Cells[CellAA][2] / (Cells[CellAA][2] + Cells[CellD][2] + ltest);
									Bonds[BondA][5] = 1.;

									// if some part of bond is hidden in tumor-dividing-brother-cell, we need to account for it
									if (ltest > 0.)
										for (int iiij = 0; iiij <= 1; iiij++)
										{
											if (iiij == 0)
												CellE = CellD;
											else
												CellE = CellAA;

											if (CellsTypes[CellE] == 0)
											{
												CellBB = TumorCellsPair[CellE];
												if (iiij == 0)
												{
													begx = Bonds[BondA][1];
													begy = Bonds[BondA][2];
													endx = Bonds[BondA][3];
													endy = Bonds[BondA][4];
												}
												else
												{
													begx = Bonds[BondA][3];
													begy = Bonds[BondA][4];
													endx = Bonds[BondA][1];
													endy = Bonds[BondA][2];
												}

												L = sqrt((begx - Cells[CellBB][0])*(begx - Cells[CellBB][0]) + (begy - Cells[CellBB][1])*(begy - Cells[CellBB][1]));

												alpha = acos(((endx - begx)*(Cells[CellBB][0] - begx) + (endy - begy)*(Cells[CellBB][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
												if (alpha < PI / 2.)
													L = L*sin(alpha);

												if (L < Cells[CellBB][2])
												{
													D = sqrt((Cells[CellE][0] - Cells[CellBB][0])*(Cells[CellE][0] - Cells[CellBB][0]) + (Cells[CellE][1] - Cells[CellBB][1])*(Cells[CellE][1] - Cells[CellBB][1]));

													if (iiij == 0)
														cosbeta = ((Bonds[BondA][3] - Bonds[BondA][1])*(Cells[CellBB][0] - Cells[CellE][0]) + (Bonds[BondA][4] - Bonds[BondA][2])*(Cells[CellBB][1] - Cells[CellE][1])) / (ltest * D);
													else
														cosbeta = ((Bonds[BondA][1] - Bonds[BondA][3])*(Cells[CellBB][0] - Cells[CellE][0]) + (Bonds[BondA][2] - Bonds[BondA][4])*(Cells[CellBB][1] - Cells[CellE][1])) / (ltest * D);

													if (D*D*(cosbeta*cosbeta - 1.) + Cells[CellBB][2] * Cells[CellBB][2] > 0.) // just in case, numerical errors are possible
														LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[CellBB][2] * Cells[CellBB][2]) - Cells[CellE][2]; // "hidden" length of bond
													else
														LH = 0.;

													if (LH > 0.) // just in case, numerical errors are possible
													{
														Bonds[BondA][0] = Bonds[BondA][0] - LH; // effective length 
														Bonds[BondA][5] = Bonds[BondA][5] * (1. + ((1. - 2.*Cells[CellE][3]) / Cells[CellE][3]) *(LH / D));
														if (BondsCells[BondA][2] == 1)
															BondsCells[BondA][2] = 3 - iiij;
														else if (BondsCells[BondA][2] > 1)
															BondsCells[BondA][2] = 4;
													}
												}
											}
										}
								}

								N_Bonds++;
								if (N_Bonds >= BondsMaxN - 10)
								{
									i = IncreaseBondsMaxN(i);
									N_BondsOld2 = N_Bonds; // this actually includes bonds formed during this cycle, but that's ok
								}
							}
							l++;
						}

						// daughter cell copies mother's potential bonds
						l = 0;
						while (CellsPotConnected[CellC][l] != -1)
						{
							ll = 0;
							while (CellsPotConnected[N_Cells][ll] != -1)
								ll++;
							if (ll >= 5 * MaxBondsForCell - 10)
								IncreaseMaxBondsForCell();
							CellsPotConnected[N_Cells][ll] = CellsPotConnected[CellC][l];

							ll = 0;
							while (CellsPotConnected[CellsPotConnected[CellC][l]][ll] != -1)
								ll++;
							if (ll >= 5 * MaxBondsForCell - 10)
								IncreaseMaxBondsForCell();
							CellsPotConnected[CellsPotConnected[CellC][l]][ll] = N_Cells;

							l++;
						}

						N_Cells++;
						if (N_Cells >= CellsMaxN - 10)
							IncreaseCellsMaxN();
					}

					// initial division bond changes type
					BondsCells[i][2] = 1; // may be changed further

										  // we need to bond all new cells with each other
					for (int ij = 1; ij <= 4; ij++)
					{
						if (ij == 1)
						{
							CellAA = CellA;
							CellBB = CellB;
							BondA = i; // rebonding
						}
						else if (ij == 2)
						{
							CellAA = CellA;
							CellBB = N_Cells - 1;
							BondA = N_Bonds;
						}
						else if (ij == 3)
						{
							CellAA = CellB;
							CellBB = N_Cells - 2;
							BondA = N_Bonds;
						}
						else if (ij == 4)
						{
							CellAA = N_Cells - 1;
							CellBB = N_Cells - 2;
							BondA = N_Bonds;
						}

						if (ij > 1)
						{
							ll = 0;
							while (CellsConnectedWith[CellAA][ll][0] != -1)
								ll++;
							if (ll >= MaxBondsForCell - 10)
								IncreaseMaxBondsForCell();
							CellsConnectedWith[CellAA][ll][0] = CellBB;
							CellsConnectedWith[CellAA][ll][1] = BondA;

							ll = 0;
							while (CellsConnectedWith[CellBB][ll][0] != -1)
								ll++;
							CellsConnectedWith[CellBB][ll][0] = CellAA;
							CellsConnectedWith[CellBB][ll][1] = BondA;

							BondsCells[BondA][0] = CellAA;
							BondsCells[BondA][1] = CellBB;
							BondsCells[BondA][2] = 1; // may be changed further
						}

						// length is this, if there are no hidden parts -- in this case will be rewritten below
						ltest = sqrt((Cells[CellAA][0] - Cells[CellBB][0])*(Cells[CellAA][0] - Cells[CellBB][0]) + (Cells[CellAA][1] - Cells[CellBB][1])*(Cells[CellAA][1] - Cells[CellBB][1])) - Cells[CellAA][2] - Cells[CellBB][2];
						Bonds[BondA][0] = ltest;
						// positions are these, if not broken -- in this case will be rewritten below
						Bonds[BondA][1] = Cells[CellAA][0] + (Cells[CellBB][0] - Cells[CellAA][0])*Cells[CellAA][2] / (Cells[CellAA][2] + Cells[CellBB][2] + Bonds[BondA][0]);
						Bonds[BondA][2] = Cells[CellAA][1] + (Cells[CellBB][1] - Cells[CellAA][1])*Cells[CellAA][2] / (Cells[CellAA][2] + Cells[CellBB][2] + Bonds[BondA][0]);
						Bonds[BondA][3] = Cells[CellBB][0] + (Cells[CellAA][0] - Cells[CellBB][0])*Cells[CellBB][2] / (Cells[CellBB][2] + Cells[CellAA][2] + Bonds[BondA][0]);
						Bonds[BondA][4] = Cells[CellBB][1] + (Cells[CellAA][1] - Cells[CellBB][1])*Cells[CellBB][2] / (Cells[CellBB][2] + Cells[CellAA][2] + Bonds[BondA][0]);
						Bonds[BondA][5] = 1.;

						// if some part of bond is hidden in tumor-dividing-brother-cell, we need to account for it
						if (ltest > 0.)
							for (int iii = 0; iii <= 1; iii++)
							{
								if (iii == 0)
									CellD = CellAA;
								else
									CellD = CellBB;

								if (CellsTypes[CellD] == 0)
								{
									CellCC = TumorCellsPair[CellD];
									if (iii == 0)
									{
										begx = Bonds[BondA][1];
										begy = Bonds[BondA][2];
										endx = Bonds[BondA][3];
										endy = Bonds[BondA][4];
									}
									else
									{
										begx = Bonds[BondA][3];
										begy = Bonds[BondA][4];
										endx = Bonds[BondA][1];
										endy = Bonds[BondA][2];
									}

									L = sqrt((begx - Cells[CellCC][0])*(begx - Cells[CellCC][0]) + (begy - Cells[CellCC][1])*(begy - Cells[CellCC][1]));

									alpha = acos(((endx - begx)*(Cells[CellCC][0] - begx) + (endy - begy)*(Cells[CellCC][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
									if (alpha < PI / 2.)
										L = L*sin(alpha);

									if (L < Cells[CellCC][2])
									{
										D = sqrt((Cells[CellD][0] - Cells[CellCC][0])*(Cells[CellD][0] - Cells[CellCC][0]) + (Cells[CellD][1] - Cells[CellCC][1])*(Cells[CellD][1] - Cells[CellCC][1]));

										if (iii == 0)
											cosbeta = ((Bonds[BondA][3] - Bonds[BondA][1])*(Cells[CellCC][0] - Cells[CellD][0]) + (Bonds[BondA][4] - Bonds[BondA][2])*(Cells[CellCC][1] - Cells[CellD][1])) / (ltest * D);
										else
											cosbeta = ((Bonds[BondA][1] - Bonds[BondA][3])*(Cells[CellCC][0] - Cells[CellD][0]) + (Bonds[BondA][2] - Bonds[BondA][4])*(Cells[CellCC][1] - Cells[CellD][1])) / (ltest * D);

										if (D*D*(cosbeta*cosbeta - 1.) + Cells[CellCC][2] * Cells[CellCC][2] > 0.) // just in case, numerical errors are possible
											LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[CellCC][2] * Cells[CellCC][2]) - Cells[CellD][2]; // "hidden" length of bond
										else
											LH = 0.;

										if (LH > 0.) // just in case, numerical errors are possible
										{
											Bonds[BondA][0] = Bonds[BondA][0] - LH; // effective length 
											Bonds[BondA][5] = Bonds[BondA][5] * (1. + ((1. - 2.*Cells[CellD][3]) / Cells[CellD][3]) *(LH / D));
											if (BondsCells[BondA][2] == 1)
												BondsCells[BondA][2] = 3 - iii;
											else if (BondsCells[BondA][2] > 1)
												BondsCells[BondA][2] = 4;
										}
									}
								}
							}

						if (ij > 1)
						{
							N_Bonds++;
							if (N_Bonds >= BondsMaxN - 10)
							{
								i = IncreaseBondsMaxN(i);
								N_BondsOld2 = N_Bonds; // this actually includes bonds formed during this cycle, but that's ok
							}
						}
					}

					// check bonds with capsules					
					for (int CellI = 1 - min(1, norm_capsule); CellI <= tumor_capsule; CellI++) // one of capsules
					{
						for (int iii = 0; iii <= 1; iii++)
						{
							if (iii == 0)
								CellD = CellA;
							else
								CellD = CellB;

							DontAccount = 1;

							lll = 0;

							while (CellsConnectedWith[CellD][lll][0] != -1)
							{
								if (CellsConnectedWith[CellD][lll][0] == CellI)
								{
									l = lll;
									DontAccount = 0;
								}
								lll++;
							}

							if (DontAccount == 0)
							{
								CellC = TumorCellsPair[CellD];
								BondA = CellsConnectedWith[CellD][l][1];

								ltest = Cells[CellI][2] - sqrt((Cells[CellD][0])*(Cells[CellD][0]) + (Cells[CellD][1])*(Cells[CellD][1])) - Cells[CellD][2];
								Bonds[BondA][0] = ltest;
								Bonds[BondA][1] = Cells[CellD][0] + Cells[CellD][2] * Cells[CellD][0] / sqrt(Cells[CellD][0] * Cells[CellD][0] + Cells[CellD][1] * Cells[CellD][1]);
								Bonds[BondA][2] = Cells[CellD][1] + Cells[CellD][2] * Cells[CellD][1] / sqrt(Cells[CellD][0] * Cells[CellD][0] + Cells[CellD][1] * Cells[CellD][1]);
								Bonds[BondA][3] = Cells[CellD][0] + (Cells[CellD][2] + Bonds[BondA][0]) * Cells[CellD][0] / sqrt(Cells[CellD][0] * Cells[CellD][0] + Cells[CellD][1] * Cells[CellD][1]);
								Bonds[BondA][4] = Cells[CellD][1] + (Cells[CellD][2] + Bonds[BondA][0]) * Cells[CellD][1] / sqrt(Cells[CellD][0] * Cells[CellD][0] + Cells[CellD][1] * Cells[CellD][1]);

								if (CellI == 0)
									BondsCells[BondA][2] = 5; // yet
								if (CellI == 1)
									BondsCells[BondA][2] = 7;

								Bonds[BondA][5] = 1.;

								begx = Bonds[BondA][1];
								begy = Bonds[BondA][2];
								endx = Bonds[BondA][3];
								endy = Bonds[BondA][4];

								if (ltest > 0.)
								{
									L = sqrt((begx - Cells[CellC][0])*(begx - Cells[CellC][0]) + (begy - Cells[CellC][1])*(begy - Cells[CellC][1]));

									alpha = acos(((endx - begx)*(Cells[CellC][0] - begx) + (endy - begy)*(Cells[CellC][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
									if (alpha < PI / 2.)
										L = L*sin(alpha);

									if (L < Cells[CellC][2])
									{
										D = sqrt((Cells[CellD][0] - Cells[CellC][0])*(Cells[CellD][0] - Cells[CellC][0]) + (Cells[CellD][1] - Cells[CellC][1])*(Cells[CellD][1] - Cells[CellC][1]));

										cosbeta = ((Bonds[BondA][3] - Bonds[BondA][1])*(Cells[CellC][0] - Cells[CellD][0]) + (Bonds[BondA][4] - Bonds[BondA][2])*(Cells[CellC][1] - Cells[CellD][1])) / (ltest * D);

										if (D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2] > 0.) // just in case, numerical errors are possible
											LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2]) - Cells[CellD][2]; // "hidden" length of bond
										else
											LH = 0.;

										if (LH > 0.) // just in case, numerical errors are possible
										{
											Bonds[BondA][0] = Bonds[BondA][0] - LH; // effective length 
											Bonds[BondA][5] = (1. + ((1. - 2.*Cells[CellD][3]) / Cells[CellD][3]) *(LH / D));
											BondsCells[BondA][2]++;
										}
									}
								}

								// and bond CellC with capsule
								ll = 0;
								while (CapsulesConnectedWith[CellI][ll][0] != -1)
									ll++;
								if (ll >= MaxBondsForCapsule - 10)
									IncreaseMaxBondsForCapsule();
								CapsulesConnectedWith[CellI][ll][0] = CellC;
								CapsulesConnectedWith[CellI][ll][1] = N_Bonds;

								ll = 0;
								while (CellsConnectedWith[CellC][ll][0] != -1)
									ll++;
								if (ll >= MaxBondsForCell - 10)
									IncreaseMaxBondsForCell();
								CellsConnectedWith[CellC][ll][0] = CellI;
								CellsConnectedWith[CellC][ll][1] = N_Bonds;

								BondsCells[N_Bonds][0] = CellC;
								BondsCells[N_Bonds][1] = CellI;

								ltest = Cells[CellI][2] - sqrt((Cells[CellC][0])*(Cells[CellC][0]) + (Cells[CellC][1])*(Cells[CellC][1])) - Cells[CellC][2];
								Bonds[N_Bonds][0] = ltest;
								Bonds[N_Bonds][1] = Cells[CellC][0] + Cells[CellC][2] * Cells[CellC][0] / sqrt(Cells[CellC][0] * Cells[CellC][0] + Cells[CellC][1] * Cells[CellC][1]);
								Bonds[N_Bonds][2] = Cells[CellC][1] + Cells[CellC][2] * Cells[CellC][1] / sqrt(Cells[CellC][0] * Cells[CellC][0] + Cells[CellC][1] * Cells[CellC][1]);
								Bonds[N_Bonds][3] = Cells[CellC][0] + (Cells[CellC][2] + Bonds[N_Bonds][0]) * Cells[CellC][0] / sqrt(Cells[CellC][0] * Cells[CellC][0] + Cells[CellC][1] * Cells[CellC][1]);
								Bonds[N_Bonds][4] = Cells[CellC][1] + (Cells[CellC][2] + Bonds[N_Bonds][0]) * Cells[CellC][1] / sqrt(Cells[CellC][0] * Cells[CellC][0] + Cells[CellC][1] * Cells[CellC][1]);

								if (CellI == 0)
									BondsCells[N_Bonds][2] = 5; // yet
								if (CellI == 1)
									BondsCells[N_Bonds][2] = 7;

								Bonds[N_Bonds][5] = 1.;

								begx = Bonds[N_Bonds][1];
								begy = Bonds[N_Bonds][2];
								endx = Bonds[N_Bonds][3];
								endy = Bonds[N_Bonds][4];

								if (ltest > 0.)
								{
									L = sqrt((begx - Cells[CellD][0])*(begx - Cells[CellD][0]) + (begy - Cells[CellD][1])*(begy - Cells[CellD][1]));

									alpha = acos(((endx - begx)*(Cells[CellD][0] - begx) + (endy - begy)*(Cells[CellD][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
									if (alpha < PI / 2.)
										L = L*sin(alpha);

									if (L < Cells[CellD][2])
									{
										D = sqrt((Cells[CellC][0] - Cells[CellD][0])*(Cells[CellC][0] - Cells[CellD][0]) + (Cells[CellC][1] - Cells[CellD][1])*(Cells[CellC][1] - Cells[CellD][1]));

										cosbeta = ((Bonds[N_Bonds][3] - Bonds[BondA][1])*(Cells[CellD][0] - Cells[CellC][0]) + (Bonds[N_Bonds][4] - Bonds[BondA][2])*(Cells[CellD][1] - Cells[CellC][1])) / (ltest * D);

										if (D*D*(cosbeta*cosbeta - 1.) + Cells[CellD][2] * Cells[CellD][2] > 0.) // just in case, numerical errors are possible
											LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[CellD][2] * Cells[CellD][2]) - Cells[CellC][2]; // "hidden" length of bond
										else
											LH = 0.;

										if (LH > 0.) // just in case, numerical errors are possible
										{
											Bonds[N_Bonds][0] = Bonds[N_Bonds][0] - LH; // effective length 
											Bonds[N_Bonds][5] = (1. + ((1. - 2.*Cells[CellC][3]) / Cells[CellC][3]) *(LH / D));
											BondsCells[N_Bonds][2]++;
										}
									}
								}
								N_Bonds++;
								if (N_Bonds >= BondsMaxN - 10)
								{
									i = IncreaseBondsMaxN(i);
									N_BondsOld2 = N_Bonds; // this actually includes bonds formed during this cycle, but that's ok
								}
							}
						}
					}
				}
			}

		if (outputEveryAction == 1)
		{
			cout << "\n CELLS DIVIDED \n";
			Output();
			system("PAUSE");
		}

		//control tau dynamically
		if (Adjust_tau == 1 && N_Cells > N_Cells_Init - 4) // to allow first divisions 
		{
			// decrease
			if (WorstShrink < 0.)
				tau = max(0.5*tau, tauMin);
			else if (WorstShrink < 0.4)
				tau = max(WorstShrink*WorstShrink*WorstShrink*tau, tauMin);
			else if (WorstShrink < 0.6)
				tau = max(WorstShrink*WorstShrink*tau, tauMin);
			else if (WorstShrink < 0.8)
				tau = max(WorstShrink*tau, tauMin);

			//	if (WorstShrink < 0.8)
			//		cout << "\ntau=" << tau << "\n";

			// increase
			if (WorstShrink > 0.99 && WorstShrink < 1. && tau < tauMax)
			{
				tau = min(tauMax, tau + (0.001*(WorstShrink - 0.99) / (1. - 0.99))*tau);
				//	cout << "\ntau=" << tau << "\n";
			}
		}

		// Checking for new bonds formation based on potential bonds
		if (t > tnl - 0.000000000000000000001)
		{
			// cells with cells + normal cells with tumor capsule
			for (int i = 2; i <= N_Cells - 1; i++)
			{
				if (CellsTypes[i] != -1)
				{
					l = 0;
					while (CellsPotConnected[i][l] != -1)
					{
						CellA = i;
						CellB = CellsPotConnected[i][l];

						if (i > CellsPotConnected[i][l])
						{
							CellA = i;
							CellB = CellsPotConnected[i][l];
							ltest = sqrt((Cells[CellA][0] - Cells[CellB][0])*(Cells[CellA][0] - Cells[CellB][0]) + (Cells[CellA][1] - Cells[CellB][1])*(Cells[CellA][1] - Cells[CellB][1])) - Cells[CellB][2] - Cells[CellA][2];
							ltest2 = ltest;

							BondsCells[N_Bonds][2] = 1; // yet

														// if at least one of cells is tumor cell, this may not be ltest we are looking for
							if ((CellsTypes[CellA] == 0 || CellsTypes[CellB] == 0) && ltest >0.)
							{
								// positions are these (if bond exists)
								l1test = Cells[CellA][0] + (Cells[CellB][0] - Cells[CellA][0])*Cells[CellA][2] / (Cells[CellA][2] + Cells[CellB][2] + ltest);
								l2test = Cells[CellA][1] + (Cells[CellB][1] - Cells[CellA][1])*Cells[CellA][2] / (Cells[CellA][2] + Cells[CellB][2] + ltest);
								l3test = Cells[CellB][0] + (Cells[CellA][0] - Cells[CellB][0])*Cells[CellB][2] / (Cells[CellB][2] + Cells[CellA][2] + ltest);
								l4test = Cells[CellB][1] + (Cells[CellA][1] - Cells[CellB][1])*Cells[CellB][2] / (Cells[CellB][2] + Cells[CellA][2] + ltest);
								l5test = 1.;

								for (int iii = 0; iii <= 1; iii++)
								{
									if (iii == 0)
										CellD = CellA;
									else
										CellD = CellB;

									if (CellsTypes[CellD] == 0)
									{
										CellC = TumorCellsPair[CellD];

										if (iii == 0)
										{
											begx = l3test;
											begy = l4test;
											endx = l1test;
											endy = l2test;
										}
										else
										{
											begx = l1test;
											begy = l2test;
											endx = l3test;
											endy = l4test;
										}

										if (ltest > 0.)
										{
											L = sqrt((begx - Cells[CellC][0])*(begx - Cells[CellC][0]) + (begy - Cells[CellC][1])*(begy - Cells[CellC][1]));

											alpha = acos(((endx - begx)*(Cells[CellC][0] - begx) + (endy - begy)*(Cells[CellC][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
											if (alpha < PI / 2.)
												L = L*sin(alpha);

											if (L < Cells[CellC][2])
											{
												D = sqrt((Cells[CellD][0] - Cells[CellC][0])*(Cells[CellD][0] - Cells[CellC][0]) + (Cells[CellD][1] - Cells[CellC][1])*(Cells[CellD][1] - Cells[CellC][1]));

												if (iii == 0)
													cosbeta = ((l3test - l1test)*(Cells[CellC][0] - Cells[CellD][0]) + (l4test - l2test)*(Cells[CellC][1] - Cells[CellD][1])) / (ltest * D);
												else
													cosbeta = ((l1test - l3test)*(Cells[CellC][0] - Cells[CellD][0]) + (l2test - l4test)*(Cells[CellC][1] - Cells[CellD][1])) / (ltest * D);

												if (D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2] > 0.) // just in case, numerical errors are possible
													LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2]) - Cells[CellD][2]; // "hidden" length of bond
												else
													LH = 0.;

												if (LH > 0.) // just in case, numerical errors are possible
												{
													ltest2 = ltest2 - LH; // effective length 
													l5test = l5test * (1. + ((1. - 2.*Cells[CellD][3]) / Cells[CellD][3]) *(LH / D));
													if (BondsCells[N_Bonds][2] == 1)
														BondsCells[N_Bonds][2] = 3 - iii;
													else if (BondsCells[N_Bonds][2] > 1)
														BondsCells[N_Bonds][2] = 4;
												}
											}
										}
									}
								}
							}
							// now we know ltest in all cases

							if (ltest2 < lbond && !(tumor_capsule == 1 && CellsTypes[CellA] + CellsTypes[CellB] == 1)) // bond is formed
							{
								// write down this bond
								ll = 0;
								while (CellsConnectedWith[CellA][ll][0] != -1)
									ll++;
								if (ll >= MaxBondsForCell - 10)
									IncreaseMaxBondsForCell();
								CellsConnectedWith[CellA][ll][0] = CellB;
								CellsConnectedWith[CellA][ll][1] = N_Bonds;

								if (CellB == 1) // it's tumor capsule
								{
									ll = 0;
									while (CapsulesConnectedWith[CellB][ll][0] != -1)
										ll++;
									if (ll >= MaxBondsForCapsule - 10)
										IncreaseMaxBondsForCapsule();
									CapsulesConnectedWith[CellB][ll][0] = CellA;
									CapsulesConnectedWith[CellB][ll][1] = N_Bonds;
								}
								else
								{
									ll = 0;
									while (CellsConnectedWith[CellB][ll][0] != -1)
										ll++;
									if (ll >= MaxBondsForCell - 10)
										IncreaseMaxBondsForCell();
									CellsConnectedWith[CellB][ll][0] = CellA;
									CellsConnectedWith[CellB][ll][1] = N_Bonds;
								}

								Bonds[N_Bonds][0] = ltest2;

								if (CellsTypes[CellA] == 0 || CellsTypes[CellB] == 0)
								{
									Bonds[N_Bonds][1] = l1test;
									Bonds[N_Bonds][2] = l2test;
									Bonds[N_Bonds][3] = l3test;
									Bonds[N_Bonds][4] = l4test;
									Bonds[N_Bonds][5] = l5test;
								}
								else
								{
									Bonds[N_Bonds][1] = Cells[CellB][0] + (Cells[CellA][0] - Cells[CellB][0])*Cells[CellB][2] / (Cells[CellB][2] + Cells[CellA][2] + Bonds[N_Bonds][0]);
									Bonds[N_Bonds][2] = Cells[CellB][1] + (Cells[CellA][1] - Cells[CellB][1])*Cells[CellB][2] / (Cells[CellB][2] + Cells[CellA][2] + Bonds[N_Bonds][0]);
									Bonds[N_Bonds][3] = Cells[CellA][0] + (Cells[CellB][0] - Cells[CellA][0])*Cells[CellA][2] / (Cells[CellB][2] + Cells[CellA][2] + Bonds[N_Bonds][0]);
									Bonds[N_Bonds][4] = Cells[CellA][1] + (Cells[CellB][1] - Cells[CellA][1])*Cells[CellA][2] / (Cells[CellB][2] + Cells[CellA][2] + Bonds[N_Bonds][0]);
									Bonds[N_Bonds][5] = 1.;
								}

								BondsCells[N_Bonds][0] = CellB;
								BondsCells[N_Bonds][1] = CellA;

								// delete this bond from potential bonds
								ll = 0;
								while (CellsPotConnected[CellA][ll] != CellB)
									ll++;
								while (CellsPotConnected[CellA][ll] != -1)
								{
									CellsPotConnected[CellA][ll] = CellsPotConnected[CellA][ll + 1];
									ll++;
								}

								if (CellB > 1)
								{
									ll = 0;
									while (CellsPotConnected[CellB][ll] != CellA)
										ll++;
									while (CellsPotConnected[CellB][ll] != -1)
									{
										CellsPotConnected[CellB][ll] = CellsPotConnected[CellB][ll + 1];
										ll++;
									}
								}

								N_Bonds++;
								if (N_Bonds >= BondsMaxN - 10)
									IL = IncreaseBondsMaxN(IL);
							}
							else if (ltest2 > lpotbr) // no need to account for this potential bond further
							{
								ll = l;
								while (CellsPotConnected[CellA][ll] != -1)
								{
									CellsPotConnected[CellA][ll] = CellsPotConnected[CellA][ll + 1];
									ll++;
								}
								l--;

								if (CellB > 1)
								{
									ll = 0;
									while (CellsPotConnected[CellB][ll] != CellA)
										ll++;
									while (CellsPotConnected[CellB][ll] != -1)
									{
										CellsPotConnected[CellB][ll] = CellsPotConnected[CellB][ll + 1];
										ll++;
									}
								}
							}
						}
						l++;
					}
				}
			}

			// cells with capsules (this is important, so no potential bonds exist -- this is checked often)
			for (int iii = 1 - min(1, norm_capsule); iii <= tumor_capsule; iii++)
			{
				for (int j = 2; j <= N_Cells - 1; j++)
				{
					if (!(iii == 1 && CellsTypes[j] == 1) && !(tumor_capsule == 1 && CellsTypes[j] == 0 && iii == 0))
					{
						DontAccount = 0;

						lll = 0;
						while (CellsConnectedWith[j][lll][0] != -1)
						{
							if (CellsConnectedWith[j][lll][0] == iii)
								DontAccount = 1;
							lll++;
						}

						if (DontAccount == 0)
						{
							ltest = Cells[iii][2] - sqrt((Cells[j][0])*(Cells[j][0]) + (Cells[j][1])*(Cells[j][1])) - Cells[j][2];
							ltest2 = ltest;

							// define type of bond
							if (iii == 0)
								BondsCells[N_Bonds][2] = 5; // yet
							if (iii == 1)
								BondsCells[N_Bonds][2] = 7;

							l5test = 1.;

							// if cell is tumor cell, this may not be ltest we are looking for
							if (CellsTypes[j] == 0 && ltest > 0.)
							{
								// positions are these (if bond exists)
								l1test = Cells[j][0] + Cells[j][2] * Cells[j][0] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
								l2test = Cells[j][1] + Cells[j][2] * Cells[j][1] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
								l3test = Cells[j][0] + (Cells[j][2] + ltest) * Cells[j][0] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
								l4test = Cells[j][1] + (Cells[j][2] + ltest) * Cells[j][1] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);

								CellD = j;

								CellC = TumorCellsPair[CellD];
								begx = l1test;
								begy = l2test;
								endx = l3test;
								endy = l4test;

								L = sqrt((begx - Cells[CellC][0])*(begx - Cells[CellC][0]) + (begy - Cells[CellC][1])*(begy - Cells[CellC][1]));

								alpha = acos(((endx - begx)*(Cells[CellC][0] - begx) + (endy - begy)*(Cells[CellC][1] - begy)) / (sqrt((begx - endx)*(begx - endx) + (begy - endy)*(begy - endy))*L));
								if (alpha < PI / 2.)
									L = L*sin(alpha);

								if (L < Cells[CellC][2])
								{
									D = sqrt((Cells[CellD][0] - Cells[CellC][0])*(Cells[CellD][0] - Cells[CellC][0]) + (Cells[CellD][1] - Cells[CellC][1])*(Cells[CellD][1] - Cells[CellC][1]));

									cosbeta = ((l3test - l1test)*(Cells[CellC][0] - Cells[CellD][0]) + (l4test - l2test)*(Cells[CellC][1] - Cells[CellD][1])) / (ltest * D);

									if (D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2]) // just in case, numerical errors are possible
										LH = D*cosbeta + sqrt(D*D*(cosbeta*cosbeta - 1.) + Cells[CellC][2] * Cells[CellC][2]) - Cells[CellD][2]; // "hidden" length of bond
									else
										LH = 0.;

									if (LH > 0.)
									{
										ltest2 = ltest2 - LH; // effective length 
										l5test = l5test * (1. + ((1. - 2.*Cells[CellD][3]) / Cells[CellD][3]) *(LH / D));
										BondsCells[N_Bonds][2]++;
									}
								}
							}
							// now we know ltest in all cases

							if (ltest2 < 1.01*lbond) // due to numerical errors it is better to do like this
							{
								if (!(CellsTypes[j] == 0 && ltest > 0.))
								{
									l1test = Cells[j][0] + Cells[j][2] * Cells[j][0] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
									l2test = Cells[j][1] + Cells[j][2] * Cells[j][1] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
									l3test = Cells[j][0] + (Cells[j][2] + ltest) * Cells[j][0] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
									l4test = Cells[j][1] + (Cells[j][2] + ltest) * Cells[j][1] / sqrt(Cells[j][0] * Cells[j][0] + Cells[j][1] * Cells[j][1]);
								}

								l = 0;
								while (CapsulesConnectedWith[iii][l][0] != -1)
									l++;
								if (l >= MaxBondsForCapsule - 10)
									IncreaseMaxBondsForCapsule();
								CapsulesConnectedWith[iii][l][0] = j;
								CapsulesConnectedWith[iii][l][1] = N_Bonds;

								l = 0;
								while (CellsConnectedWith[j][l][0] != -1)
									l++;
								if (l >= MaxBondsForCell - 10)
									IncreaseMaxBondsForCell();
								CellsConnectedWith[j][l][0] = iii;
								CellsConnectedWith[j][l][1] = N_Bonds;

								BondsCells[N_Bonds][0] = j;
								BondsCells[N_Bonds][1] = iii;

								Bonds[N_Bonds][0] = ltest2;
								Bonds[N_Bonds][1] = l1test;
								Bonds[N_Bonds][2] = l2test;
								Bonds[N_Bonds][3] = l3test;
								Bonds[N_Bonds][4] = l4test;
								Bonds[N_Bonds][5] = l5test;

								N_Bonds++;
								if (N_Bonds >= BondsMaxN - 10)
									IL = IncreaseBondsMaxN(IL);
							}
						}
					}
				}
			}
			tnl = tnl + Tbond;
		}

		if (outputEveryAction == 1)
		{
			cout << "\n NEW BONDS FORMED \n";
			Output();
			system("PAUSE");
		}

		// Checking for potential bonds
		if (t > tnlr - 0.000000000000000000001)
		{
			for (int i = 2; i <= N_Cells - 1; i++)
			{
				if (CellsTypes[i] != -1)
				{
					for (int j = 1; j <= i - 1; j++)
						if (CellsTypes[j] != -1)// && !(tumor_capsule == 1 && j == 1 && CellsTypes[i] == 1))
						{
							ltest = sqrt((Cells[j][0] - Cells[i][0])*(Cells[j][0] - Cells[i][0]) + (Cells[j][1] - Cells[i][1])*(Cells[j][1] - Cells[i][1])) - Cells[j][2] - Cells[i][2];

							if (ltest < lpot && ltest > -2.*r0)
							{
								DontAccount = 0;

								l = 0;
								while (CellsConnectedWith[i][l][0] != -1)
								{
									if (CellsConnectedWith[i][l][0] == j)
										DontAccount = 1;
									l++;
								}

								l = 0;
								while (CellsPotConnected[i][l] != -1)
								{
									if (CellsPotConnected[i][l] == j)
										DontAccount = 1;
									l++;
								}

								if (TumorCellsPair[i] == j)
									DontAccount = 1;

								if (DontAccount == 0)
								{
									if (l >= 5 * MaxBondsForCell - 10)
										IncreaseMaxBondsForCell();
									CellsPotConnected[i][l] = j;

									if (j > 1)
									{
										ll = 0;
										while (CellsPotConnected[j][ll] != -1)
											ll++;
										if (l >= 5 * MaxBondsForCell - 10)
											IncreaseMaxBondsForCell();
										CellsPotConnected[j][ll] = i;
									}
								}
							}
						}
				}
			}

			//and defining which normal cells supply nutrient
			if (layers > 0 && Nutr == 1)
			{
				for (int i = 0; i <= N_Cells_Init - 1; i++)
				{
					NormalCellsToCheck[i] = -1;
					NormalCellsToCheck2[i] = -1;
				}

				for (int i = 4; i <= N_Cells_Init - 1; i++)
					if (CellsTypes[i] == 1) // normal cell
						Nutrient[i][3] = 0.;

				// normal cells, connected to normal capsule, supply nutrient
				l = 0;
				while (CapsulesConnectedWith[0][l][0] != -1)
				{
					if (CellsTypes[CapsulesConnectedWith[0][l][0]] == 1)
					{
						Nutrient[CapsulesConnectedWith[0][l][0]][3] = 1.;
						NormalCellsToCheck[l] = CapsulesConnectedWith[0][l][0];
					}
					l++;
				}

				// normal cells, connected to them, also do
				StopIt = 0;

				while (StopIt == 0)
				{
					StopIt = 1;
					l = 0;
					while (NormalCellsToCheck[l] != -1)
					{
						ll = 0;
						while (CellsConnectedWith[NormalCellsToCheck[l]][ll][0] != -1)
						{
							if (CellsTypes[CellsConnectedWith[NormalCellsToCheck[l]][ll][0]] == 1)
							{
								lll = 0; // is it already accounted for
								while (NormalCellsToCheck2[lll] != -1 && NormalCellsToCheck2[lll] != CellsConnectedWith[NormalCellsToCheck[l]][ll][0])
									lll++;

								if (NormalCellsToCheck2[lll] == -1 && Nutrient[CellsConnectedWith[NormalCellsToCheck[l]][ll][0]][3] == 0.)
								{
									NormalCellsToCheck2[lll] = CellsConnectedWith[NormalCellsToCheck[l]][ll][0];
									Nutrient[CellsConnectedWith[NormalCellsToCheck[l]][ll][0]][3] = 1.;
									StopIt = 0;
								}
							}
							ll++;
						}
						l++;
					}
					for (int i = 0; i <= N_Cells_Init - 1; i++)
					{
						NormalCellsToCheck[i] = NormalCellsToCheck2[i];
						NormalCellsToCheck2[i] = -1;
					}
				}
			}

			tnlr = tnlr + Tpot;
		}

		if (outputEveryAction == 1)
		{
			cout << "\n POTENTIAL BONDS FORMED \n";
			Output();
			system("PAUSE");
		}

		// Output
		if (t > tv - 0.000000000000000000001)
		{
			VolumeOutput();
			tv = tv + volume_frequency;
		}

		if (t > to - 0.000000000000000000001)
		{
			Output();
			to = to + output_frequency;
		}
	}

	if (nofo == 0)
	{
		d = -1;
		Output();
	}
	exit(0);

	// delete arrays
	delete[] PressureOnCell;

	for (int i = 0; i <= CellsMaxN - 1; i++)
		delete[] Cells[i];
	delete[] Cells;

	delete[] CellsTypes;

	delete[] CellMoveMode;

	for (int i = 0; i <= BondsMaxN - 1; i++)
		delete[] Bonds[i];
	delete[] Bonds;

	for (int i = 0; i <= BondsMaxN - 1; i++)
		delete[] BondsCells[i];
	delete[] BondsCells;

	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			delete[] CellsConnectedWith[i][j];
		delete[] CellsConnectedWith[i];
	}
	delete[] CellsConnectedWith;

	for (int i = 0; i <= 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCapsule - 1; j++)
			delete[] CapsulesConnectedWith[i][j];
		delete[] CapsulesConnectedWith[i];
	}
	delete[] CapsulesConnectedWith;

	for (int i = 0; i <= CellsMaxN - 1; i++)
		delete[] CellsPotConnected[i];
	delete[] CellsPotConnected;

	//	_getch();
}

void VolumeOutput()
{
	// note that tumor cell volume calculated here does not account for negative bonds
	ProlTumorVolume = 0.;
	QuiesTumorVolume = 0.;
	DyingTumorVolume = 0.;
	N_Alive_Tumor_Cells = 0;

	for (int i = 2; i <= N_Cells - 1; i++)
		if (CellsTypes[i] == 0 && TumorCellsPair[i] > i) // to not duplicate
		{
			N_Alive_Tumor_Cells++;

			if (CellMoveMode[i] == 1 || CellMoveMode[i] == 2) // proliferating zone
				ProlTumorVolume = ProlTumorVolume + Cells[i][3] * Cells[i][2] * Cells[i][2];
			else if (CellMoveMode[i] == 3) // dying zone
				DyingTumorVolume = DyingTumorVolume + Cells[i][3] * Cells[i][2] * Cells[i][2];
			else // quiescent zone
				QuiesTumorVolume = QuiesTumorVolume + Cells[i][3] * Cells[i][2] * Cells[i][2];
		}

	ProlTumorVolume = ProlTumorVolume*2.*PI;
	QuiesTumorVolume = QuiesTumorVolume*2.*PI;
	DyingTumorVolume = DyingTumorVolume*2.*PI;
	TumorVolume = ProlTumorVolume + QuiesTumorVolume + DyingTumorVolume;

	if (t == 0.)
	{
		QuiesTumorVolume = 0.;
		ProlTumorVolume = TumorVolume;
	}

	fprintf(volume, "%.4f\t%.7e\t%.7e\t%.7e\t%.7e\t%.7e\t%.7e\t%i\n", t, TumorVolume, Cells[1][2], Cells[0][2], ProlTumorVolume, QuiesTumorVolume, DyingTumorVolume, N_Alive_Tumor_Cells);
	fflush(volume);

	if (N_Alive_Tumor_Cells >= Nstop)
	{
		d = -1;
		Output();
		exit(0);
	}
}

void Output()
{
	// for debugging -- to check for discrepancies of bonds data
/*	for (int i = 0; i <= N_Cells - 1; i++)
	{
		l = 0;
		while (CellsConnectedWith[i][l][0] != -1)
		{
			if ((CellsConnectedWith[i][l][0]>1 && CellsConnectedWith[i][l][1] > 1) && ((BondsCells[CellsConnectedWith[i][l][1]][0] != i && BondsCells[CellsConnectedWith[i][l][1]][1] != i) || (BondsCells[CellsConnectedWith[i][l][1]][0] != CellsConnectedWith[i][l][0] && BondsCells[CellsConnectedWith[i][l][1]][1] != CellsConnectedWith[i][l][0])))
			{
				cout << "\n BOND DATA WRONG\n";
				cout << "CellsConnectedWith[" << i << "][" << l << "][0]=" << CellsConnectedWith[i][l][0] << "\n";
				cout << "CellsConnectedWith[" << i << "][" << l << "][1]=" << CellsConnectedWith[i][l][1] << "\n";
				cout << "BondsCells[" << CellsConnectedWith[i][l][1] << "][0]=" << BondsCells[CellsConnectedWith[i][l][1]][0] << "\n";
				cout << "BondsCells[" << CellsConnectedWith[i][l][1] << "][1]=" << BondsCells[CellsConnectedWith[i][l][1]][1] << "\n";
				cout << "BondsCells[" << CellsConnectedWith[i][l][1] - 1 << "][0]=" << BondsCells[CellsConnectedWith[i][l][1] - 1][0] << "\n";
				cout << "BondsCells[" << CellsConnectedWith[i][l][1] - 1 << "][1]=" << BondsCells[CellsConnectedWith[i][l][1] - 1][1] << "\n";
				cout << "BondsCells[" << CellsConnectedWith[i][l][1] + 1 << "][0]=" << BondsCells[CellsConnectedWith[i][l][1] + 1][0] << "\n";
				cout << "BondsCells[" << CellsConnectedWith[i][l][1] + 1 << "][1]=" << BondsCells[CellsConnectedWith[i][l][1] + 1][1] << "\n";
				system("PAUSE");
			}
			l++;
		}
	}

	int *DivisionCheck;
	DivisionCheck = new int[N_Cells];
	for (int i = 0; i <= N_Cells - 1; i++)
		DivisionCheck[i] = 0;

	for (int i = 0; i <= N_Bonds - 1; i++)
	{
		if (BondsCells[i][2] == 9)
		{
			DivisionCheck[BondsCells[i][0]]++;
			DivisionCheck[BondsCells[i][1]]++;

			if (DivisionCheck[BondsCells[i][0]] > 1)
			{
				cout << "\n BOND DATA WRONG\n";
				cout << "Cell " << BondsCells[i][0] << " has two dividing brothers \n";
				system("PAUSE");
			}
			if (DivisionCheck[BondsCells[i][1]] > 1)
			{
				cout << "\n BOND DATA WRONG\n";
				cout << "Cell " << BondsCells[i][1] << " has two dividing brothers \n";
				system("PAUSE");
			}
		}
	}

	delete[] DivisionCheck;
	DivisionCheck = NULL;

	for (int i = 0; i <= N_Bonds - 1; i++)
	{
		if (BondsCells[i][2] > 0)
		{
			CellA = BondsCells[i][0];
			CellB = BondsCells[i][1];

			if (CellA > 1 && CellB > 1)
			{

				l = 0;
				while (CellsConnectedWith[CellA][l][0] != CellB)
				{
					if (CellsConnectedWith[CellA][l][0] == -1)
					{
						cout << "\n BOND DATA WRONG\n";
						cout << "\n Bond not recorded in CellsConnectedWith -- " << CellA << " and " << CellB << "\n";
						system("PAUSE");
					}
					l++;
				}

				l = 0;
				while (CellsConnectedWith[CellB][l][0] != CellA)
				{
					if (CellsConnectedWith[CellB][l][0] == -1)
					{
						cout << "\n BOND DATA WRONG\n";
						cout << "\n Bond not recorded in CellsConnectedWith -- " << CellB << " and " << CellA << "\n";
						system("PAUSE");
					}
					l++;
				}
			}
		}
	}
	*/

	if (OutputFlg == 0)
		cout << "t=" << (int)(t) << "/" << T << " N=" << N_Alive_Tumor_Cells << "\n";
	else
	{
		_itoa(d, num, 10);

		// for debugging
		if (d == debugd)
		{
			output_frequency = tau;
			to = t;
		}

		if (d == debugd2)
			outputEveryAction = 1;

		if (Nstop < 99999999)
			cout << "d=" << d << "; N=" << N_Alive_Tumor_Cells << "/" << Nstop << "\n";
		else
			cout << "d=" << d << "; t=" << (int)(t) << "/" << T << "; N=" << N_Alive_Tumor_Cells << "\n";

		if (d < 10)
			str = "-00000";
		else if (d < 100)
			str = "-0000";
		else if (d < 1000)
			str = "-000";
		else if (d < 10000)
			str = "-00";
		else if (d < 100000)
			str = "-0";
		else
			str = "-";

		delete[] char_array;
		int nn = str.length();
		char * char_array = new char[nn];
		strcpy(char_array, str.c_str());

		// CELLS
		strcpy(File_name_cells, "Cells");
		strcat(File_name_cells, char_array);
		strcat(File_name_cells, num);
		strcat(File_name_cells, ".txt");

		if ((out = fopen(File_name_cells, "wt")) == NULL)
			fprintf(stderr, "Cannot open file Cells\n");

		fprintf(out, "N\tx\t\t\ty\t\t\trad\t\tfs\t\ttype\n\n");

		for (int i = 0; i <= N_Cells - 1; i++)
			if (CellsTypes[i] != -1)
				fprintf(out, "%i\t%.7e\t%.7e\t%.5f\t%.5f\t%i\n", i, Cells[i][0], Cells[i][1], Cells[i][2], Cells[i][3], CellsTypes[i]);

		fclose(out);

		// Bonds
		strcpy(File_name_Bonds, "Bonds");
		strcat(File_name_Bonds, char_array);
		strcat(File_name_Bonds, num);
		strcat(File_name_Bonds, ".txt");

		if ((out = fopen(File_name_Bonds, "wt")) == NULL)
			fprintf(stderr, "Cannot open file Bonds\n");

		fprintf(out, "i\tj\tLen\t\t\tBx\t\t\tBy\t\t\tEx\t\t\tEy\t\tkStress\ttype\n\n");

		for (int i = 1; i <= N_Bonds - 1; i++)
			if (BondsCells[i][2] > 0)
				fprintf(out, "%i\t%i\t%.7e\t%.7e\t%.7e\t%.7e\t%.7e\t%.3f\t%i\n", BondsCells[i][0], BondsCells[i][1], Bonds[i][0], Bonds[i][1], Bonds[i][2], Bonds[i][3], Bonds[i][4], Bonds[i][5], BondsCells[i][2]);

		fclose(out);

		if (layers>0 && Nutr == 1)
		{
			// nutrient
			strcpy(File_name_cells, "Nutr");
			strcat(File_name_cells, char_array);
			strcat(File_name_cells, num);
			strcat(File_name_cells, ".txt");

			if ((out = fopen(File_name_cells, "wt")) == NULL)
				fprintf(stderr, "Cannot open file nutrient\n");

			fprintf(out, "N\tx\t\t\ty\t\t\tlevel\t\t\tkB\n\n");

			for (int i = 0; i <= N_Cells - 1; i++)
				if (NutrCells[i] >= 0)
					fprintf(out, "%i\t%.7e\t%.7e\t%.7e\t%.3f\n", i, Nutrient[i][0], Nutrient[i][1], Nutrient[i][2], Nutrient[i][3]);

			// for nice pictures set nutrient values on the capsule 
			l = 0;
			while (CapsulesConnectedWith[0][l][0] != -1)
			{
				CellAA = CapsulesConnectedWith[0][l][0];

				if (NutrCells[CellAA] >= 0)
					fprintf(out, "%i\t%.7e\t%.7e\t%.7e\t%.3f\n", CellAA, Cells[CellAA][0] * (Cells[0][2] + Cells[CellAA][2]) / (sqrt(Cells[CellAA][0] * Cells[CellAA][0] + Cells[CellAA][1] * Cells[CellAA][1])), Cells[CellAA][1] * (Cells[0][2] + Cells[CellAA][2]) / (sqrt(Cells[CellAA][0] * Cells[CellAA][0] + Cells[CellAA][1] * Cells[CellAA][1])), Nutrient[CellAA][2], Nutrient[CellAA][3]);

				l++;
			}
		}
		
		fclose(out);

		d++;
	}
}

double Stress(double ll, double kk)
{
	cc = (2. * r0 / (2. * r0 + ll));
	cc = cc*cc*cc;

	if (cc < ccr)
		return 0.;
	else if (cc < cinf)
		return (min(15000., kk*(cc - c0)*(cc - ccr)*(cc - ccr) / sqrt(cinf - cc)));
	else
		return (15000.); // to avoid technical difficulties, here just any large number should be
}

void CellKill(int CK)
{
	// delete cell
	CellsTypes[CK] = -1;
	CellMoveMode[CK] = -1;
	Cells[CK][0] = 0.;
	Cells[CK][1] = 0.;
	Cells[CK][2] = -1.;
	Cells[CK][3] = -1.;
	NutrCells[CK] = -1;

	// delete its bonds and markers of bonds
	l = 0;
	while (CellsConnectedWith[CK][l][0] != -1)
	{
		BondA = CellsConnectedWith[CK][l][1];

		Bonds[BondA][0] = 0.;
		Bonds[BondA][1] = 0.;
		Bonds[BondA][2] = 0.;
		Bonds[BondA][3] = 0.;
		Bonds[BondA][4] = 0.;
		Bonds[BondA][5] = 1.;

		BondsCells[BondA][0] = 0;
		BondsCells[BondA][1] = 0;
		BondsCells[BondA][2] = 0;


		if (CellsConnectedWith[CK][l][0] <= 1) // capsule
		{
			ll = 0;
			while (CapsulesConnectedWith[CellsConnectedWith[CK][l][0]][ll][0] != CK)
				ll++;
			while (CapsulesConnectedWith[CellsConnectedWith[CK][l][0]][ll][0] != -1)
			{
				CapsulesConnectedWith[CellsConnectedWith[CK][l][0]][ll][0] = CapsulesConnectedWith[CellsConnectedWith[CK][l][0]][ll + 1][0];
				CapsulesConnectedWith[CellsConnectedWith[CK][l][0]][ll][1] = CapsulesConnectedWith[CellsConnectedWith[CK][l][0]][ll + 1][1];
				ll++;
			}
		}
		else
		{
			ll = 0;
			while (CellsConnectedWith[CellsConnectedWith[CK][l][0]][ll][0] != CK)
				ll++;
			while (CellsConnectedWith[CellsConnectedWith[CK][l][0]][ll][0] != -1)
			{
				CellsConnectedWith[CellsConnectedWith[CK][l][0]][ll][0] = CellsConnectedWith[CellsConnectedWith[CK][l][0]][ll + 1][0];
				CellsConnectedWith[CellsConnectedWith[CK][l][0]][ll][1] = CellsConnectedWith[CellsConnectedWith[CK][l][0]][ll + 1][1];
				ll++;
			}
		}

		CellsConnectedWith[CK][l][0] = -1;
		CellsConnectedWith[CK][l][1] = -1;
		l++;
	}

	// delete its markers of potential bonds
	l = 0;
	while (CellsPotConnected[CK][l] != -1)
	{
		ll = 0;
		while (CellsPotConnected[CellsPotConnected[CK][l]][ll] != CK)
			ll++;
		while (CellsPotConnected[CellsPotConnected[CK][l]][ll] != -1)
		{
			CellsPotConnected[CellsPotConnected[CK][l]][ll] = CellsPotConnected[CellsPotConnected[CK][l]][ll + 1];
			ll++;
		}
		CellsPotConnected[CK][l] = -1;
		l++;
	}

	l = 2;
	while (CellsTypes[l] == -1 && l < N_Cells)
		l++;

	if (l == N_Cells)
	{
		cout << "\n All cells are dead \n";
		system("PAUSE");
		exit(0);
	}
}

void IncreaseCellsMaxN()
{
	//cout << "\n CellsMaxN=" << CellsMaxN << "\n";
//	cout << "\n IncreaseCellsMaxN\n";

	CellsMaxNold = CellsMaxN;
	CellsMaxN = 2 * CellsMaxN;

	//create new bigger arrays

	TumorCellsPairNew = new int[CellsMaxN];
	for (int i = 0; i <= N_Cells - 1; i++)
		TumorCellsPairNew[i] = TumorCellsPair[i];
	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		TumorCellsPairNew[i] = -1;

	PressureOnCellNew = new double[CellsMaxN];
	for (int i = 0; i <= N_Cells - 1; i++)
		PressureOnCellNew[i] = PressureOnCell[i];
	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		PressureOnCellNew[i] = 0.;

	CellsNew = new double*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellsNew[i] = new double[4];
	for (int i = 0; i <= N_Cells - 1; i++)
		for (int j = 0; j <= 3; j++)
			CellsNew[i][j] = Cells[i][j];
	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 3; j++)
			CellsNew[i][j] = 0.;

	CellsTypesNew = new int[CellsMaxN];
	for (int i = 0; i <= N_Cells - 1; i++)
		CellsTypesNew[i] = CellsTypes[i];
	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		CellsTypesNew[i] = -1;

	CellMoveModeNew = new int[CellsMaxN];
	for (int i = 0; i <= N_Cells - 1; i++)
		CellMoveModeNew[i] = CellMoveMode[i];
	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		CellMoveModeNew[i] = -1;

	CellsConnectedWithNew = new int**[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		CellsConnectedWithNew[i] = new int*[MaxBondsForCell];
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			CellsConnectedWithNew[i][j] = new int[2];
	}

	for (int i = 0; i <= N_Cells - 1; i++)
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
		{
			CellsConnectedWithNew[i][j][0] = CellsConnectedWith[i][j][0];
			CellsConnectedWithNew[i][j][1] = CellsConnectedWith[i][j][1];
		}

	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
		{
			CellsConnectedWithNew[i][j][0] = -1;
			CellsConnectedWithNew[i][j][1] = -1;
		}

	CellsPotConnectedNew = new int*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellsPotConnectedNew[i] = new int[5 * MaxBondsForCell];

	for (int i = 0; i <= N_Cells - 1; i++)
		for (int j = 0; j <= 5 * MaxBondsForCell - 1; j++)
			CellsPotConnectedNew[i][j] = CellsPotConnected[i][j];
	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 5 * MaxBondsForCell - 1; j++)
			CellsPotConnectedNew[i][j] = -1;

	NutrientNew = new double*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		NutrientNew[i] = new double[4];

	for (int i = 0; i <= N_Cells - 1; i++)
		for (int j = 0; j <= 3; j++)
			NutrientNew[i][j] = Nutrient[i][j];
	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 3; j++)
			NutrientNew[i][j] = 0.;

	NutrCellsNew = new int[CellsMaxN];
	for (int i = 0; i <= N_Cells - 1; i++)
		NutrCellsNew[i] = NutrCells[i];
	for (int i = N_Cells; i <= CellsMaxN - 1; i++)
		NutrCellsNew[i] = -1;

	//delete old shorter arrays

	delete[] PressureOnCell;
	delete[] TumorCellsPair;

	for (int i = 0; i <= CellsMaxNold - 1; i++)
		delete[] Cells[i];
	delete[] Cells;

	delete[] CellsTypes;

	delete[] CellMoveMode;

	for (int i = 0; i <= CellsMaxNold - 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			delete[] CellsConnectedWith[i][j];
		delete[] CellsConnectedWith[i];
	}
	delete[] CellsConnectedWith;

	for (int i = 0; i <= CellsMaxNold - 1; i++)
		delete[] CellsPotConnected[i];
	delete[] CellsPotConnected;

	for (int i = 0; i <= CellsMaxNold - 1; i++)
		delete[] Nutrient[i];
	delete[] Nutrient;

	for (int i = 0; i <= CellsMaxNold - 1; i++)
		delete[] NutrientBu[i];
	delete[] NutrientBu;

	delete[] NutrCells;

	PressureOnCell = NULL;
	TumorCellsPair = NULL;
	Cells = NULL;
	CellsTypes = NULL;
	CellMoveMode = NULL;
	CellsConnectedWith = NULL;
	CellsPotConnected = NULL;
	Nutrient = NULL;
	NutrientBu = NULL;
	NutrCells = NULL;

	// reassign
	TumorCellsPair = new int[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		TumorCellsPair[i] = TumorCellsPairNew[i];

	PressureOnCell = new double[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		PressureOnCell[i] = PressureOnCellNew[i];

	Cells = new double*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		Cells[i] = new double[4];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 3; j++)
			Cells[i][j] = CellsNew[i][j];

	CellsTypes = new int[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellsTypes[i] = CellsTypesNew[i];

	CellMoveMode = new int[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellMoveMode[i] = CellMoveModeNew[i];

	CellsConnectedWith = new int**[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		CellsConnectedWith[i] = new int*[MaxBondsForCell];
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			CellsConnectedWith[i][j] = new int[2];
	}

	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
		{
			CellsConnectedWith[i][j][0] = CellsConnectedWithNew[i][j][0];
			CellsConnectedWith[i][j][1] = CellsConnectedWithNew[i][j][1];
		}

	CellsPotConnected = new int*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellsPotConnected[i] = new int[5 * MaxBondsForCell];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 5 * MaxBondsForCell - 1; j++)
			CellsPotConnected[i][j] = CellsPotConnectedNew[i][j];

	Nutrient = new double*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		Nutrient[i] = new double[4];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 3; j++)
			Nutrient[i][j] = NutrientNew[i][j];

	NutrientBu = new double*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		NutrientBu[i] = new double[4];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 3; j++)
			NutrientBu[i][j] = NutrientNew[i][j];

	NutrCells = new int[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		NutrCells[i] = NutrCellsNew[i];

	//delete auxiliary arrays

	delete[] PressureOnCellNew;
	delete[] TumorCellsPairNew;

	for (int i = 0; i <= CellsMaxN - 1; i++)
		delete[] CellsNew[i];
	delete[] CellsNew;

	delete[] CellsTypesNew;

	delete[] CellMoveModeNew;

	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			delete[] CellsConnectedWithNew[i][j];
		delete[] CellsConnectedWithNew[i];
	}
	delete[] CellsConnectedWithNew;

	for (int i = 0; i <= CellsMaxN - 1; i++)
		delete[] CellsPotConnectedNew[i];
	delete[] CellsPotConnectedNew;

	for (int i = 0; i <= CellsMaxN - 1; i++)
		delete[] NutrientNew[i];
	delete[] NutrientNew;

	delete[] NutrCellsNew;

	TumorCellsPairNew = NULL;
	PressureOnCellNew = NULL;
	CellsNew = NULL;
	CellsTypesNew = NULL;
	CellMoveModeNew = NULL;
	CellsConnectedWithNew = NULL;
	CellsPotConnectedNew = NULL;
	NutrientNew = NULL;
	NutrCellsNew = NULL;
//	cout << "\n Finished IncreaseCellsMaxN\n";
}

int IncreaseBondsMaxN(int li)
{
//	cout << "\n Getting memory for bonds \n";
	int liold = li;

	BondsCorr = new int[N_Bonds];

	l = 0;
	lll = 1;
	N_BondsOld = N_Bonds;

	while (lll + l <= N_BondsOld - 1)
	{
		if (BondsCells[lll + l][2] == 0)
			while (BondsCells[lll + l][2] == 0)
			{
				if (lll + l <= liold)
					li--;
				l++;
			}

		BondsCells[lll][0] = BondsCells[lll + l][0];
		BondsCells[lll][1] = BondsCells[lll + l][1];
		BondsCells[lll][2] = BondsCells[lll + l][2];
		Bonds[lll][0] = Bonds[lll + l][0];
		Bonds[lll][1] = Bonds[lll + l][1];
		Bonds[lll][2] = Bonds[lll + l][2];
		Bonds[lll][3] = Bonds[lll + l][3];
		Bonds[lll][4] = Bonds[lll + l][4];
		Bonds[lll][5] = Bonds[lll + l][5];

		BondsCorr[lll + l] = lll;
		lll++;
	}
	N_Bonds = lll;

	for (int i = N_Bonds; i <= BondsMaxN - 1; i++)
		for (int j = 0; j <= 5; j++)
			Bonds[i][j] = 0.;

	for (int i = 0; i <= N_Cells; i++)
	{
		l = 0;
		while (CellsConnectedWith[i][l][0] != -1)
		{
			CellsConnectedWith[i][l][1] = BondsCorr[CellsConnectedWith[i][l][1]];
			l++;
		}
	}

	for (int i = 0; i <= 1; i++)
	{
		l = 0;
		while (CapsulesConnectedWith[i][l][0] != -1)
		{
			CapsulesConnectedWith[i][l][1] = BondsCorr[CapsulesConnectedWith[i][l][1]];
			l++;
		}
	}

	delete[] BondsCorr;
	BondsCorr = NULL;

	l = 0; // just in case, somehow it allows to overcome one strange and non-always-repeating-and-producing-different-results bug

//	cout << "\n BondsMaxN=" << MaxBondsForCell << "\n";
//	cout << "\n IncreaseBondsMaxN\n";

	BondsMaxNold = BondsMaxN;
	BondsMaxN = 2 * BondsMaxN;

	//create new bigger arrays

	BondsNew = new double*[BondsMaxN];
	for (int i = 0; i <= BondsMaxN - 1; i++)
		BondsNew[i] = new double[6];

	for (int i = 0; i <= N_Bonds - 1; i++)
		for (int j = 0; j <= 5; j++)
			BondsNew[i][j] = Bonds[i][j];
	for (int i = N_Bonds; i <= BondsMaxN - 1; i++)
		for (int j = 0; j <= 5; j++)
			BondsNew[i][j] = 0.;

	BondsCellsNew = new int*[BondsMaxN];
	for (int i = 0; i <= BondsMaxN - 1; i++)
		BondsCellsNew[i] = new int[3];

	for (int i = 0; i <= N_Bonds - 1; i++)
		for (int j = 0; j <= 2; j++)
			BondsCellsNew[i][j] = BondsCells[i][j];
	for (int i = N_Bonds; i <= BondsMaxN - 1; i++)
		for (int j = 0; j <= 2; j++)
			BondsCellsNew[i][j] = -1;

	//delete old shorter arrays

	for (int i = 0; i <= BondsMaxNold - 1; i++)
		delete[] Bonds[i];
	delete[] Bonds;

	for (int i = 0; i <= BondsMaxNold - 1; i++)
		delete[] BondsCells[i];
	delete[] BondsCells;

	Bonds = NULL;
	BondsCells = NULL;

	// reassign

	Bonds = new double*[BondsMaxN];
	for (int i = 0; i <= BondsMaxN - 1; i++)
		Bonds[i] = new double[6];
	for (int i = 0; i <= BondsMaxN - 1; i++)
		for (int j = 0; j <= 5; j++)
			Bonds[i][j] = BondsNew[i][j];

	BondsCells = new int*[BondsMaxN];
	for (int i = 0; i <= BondsMaxN - 1; i++)
		BondsCells[i] = new int[3];
	for (int i = 0; i <= BondsMaxN - 1; i++)
		for (int j = 0; j <= 2; j++)
			BondsCells[i][j] = BondsCellsNew[i][j];

	//delete auxiliary arrays

	delete[] BondsNew;
	delete[] BondsCellsNew;

	BondsNew = NULL;
	BondsCellsNew = NULL;

//	cout << "\n Finished IncreaseBondsMaxN\n";

	return(li);
}

void IncreaseMaxBondsForCell()
{
//	cout << "\n MaxBondsForCell=" << MaxBondsForCell << "\n";
//	cout << "\n IncreaseMaxBondsForCell\n";

	MaxBondsForCellold = MaxBondsForCell;
	MaxBondsForCell = 2 * MaxBondsForCell;

	//create new bigger arrays
	CellsConnectedWithNew = new int**[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		CellsConnectedWithNew[i] = new int*[MaxBondsForCell];
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			CellsConnectedWithNew[i][j] = new int[2];
	}

	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCellold - 1; j++)
		{
			CellsConnectedWithNew[i][j][0] = CellsConnectedWith[i][j][0];
			CellsConnectedWithNew[i][j][1] = CellsConnectedWith[i][j][1];
		}
		for (int j = MaxBondsForCellold; j <= MaxBondsForCell - 1; j++)
		{
			CellsConnectedWithNew[i][j][0] = -1;
			CellsConnectedWithNew[i][j][1] = -1;
		}
	}

	CellsPotConnectedNew = new int*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellsPotConnectedNew[i] = new int[5 * MaxBondsForCell];
	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		for (int j = 0; j <= 5 * MaxBondsForCellold - 1; j++)
			CellsPotConnectedNew[i][j] = CellsPotConnected[i][j];
		for (int j = 5 * MaxBondsForCellold; j <= 5 * MaxBondsForCell - 1; j++)
			CellsPotConnectedNew[i][j] = -1;
	}

	//delete old shorter arrays

	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCellold - 1; j++)
			delete[] CellsConnectedWith[i][j];
		delete[] CellsConnectedWith[i];
	}
	delete[] CellsConnectedWith;

	for (int i = 0; i <= CellsMaxN - 1; i++)
		delete[] CellsPotConnected[i];
	delete[] CellsPotConnected;

	CellsConnectedWith = NULL;
	CellsPotConnected = NULL;

	// reassign
	CellsConnectedWith = new int**[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		CellsConnectedWith[i] = new int*[MaxBondsForCell];
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			CellsConnectedWith[i][j] = new int[2];
	}
	for (int i = 0; i <= CellsMaxN - 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
		{
			CellsConnectedWith[i][j][0] = CellsConnectedWithNew[i][j][0];
			CellsConnectedWith[i][j][1] = CellsConnectedWithNew[i][j][1];
		}
	}

	CellsPotConnected = new int*[CellsMaxN];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		CellsPotConnected[i] = new int[5 * MaxBondsForCell];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= 5 * MaxBondsForCell - 1; j++)
			CellsPotConnected[i][j] = CellsPotConnectedNew[i][j];

	//delete auxiliary arrays

	for (int i = 0; i <= CellsMaxN - 1; i++)
		for (int j = 0; j <= MaxBondsForCell - 1; j++)
			delete[] CellsConnectedWithNew[i][j];
	for (int i = 0; i <= CellsMaxN - 1; i++)
		delete[] CellsConnectedWithNew[i];
	delete[] CellsConnectedWithNew;

	for (int i = 0; i <= CellsMaxN - 1; i++)
		delete[] CellsPotConnectedNew[i];
	delete[] CellsPotConnectedNew;

	CellsConnectedWithNew = NULL;
	CellsPotConnectedNew = NULL;

//	cout << "\n Finished IncreaseMaxBondsForCell\n";
}

void IncreaseMaxBondsForCapsule()
{
//	cout << "\n MaxBondsForCapsule=" << MaxBondsForCapsule << "\n";
//	cout << "\n IncreaseMaxBondsForCapsule\n";

	MaxBondsForCapsuleold = MaxBondsForCapsule;
	MaxBondsForCapsule = 2 * MaxBondsForCapsule;

	//create new bigger arrays
	CapsulesConnectedWithNew = new int**[2];
	for (int i = 0; i <= 1; i++)
	{
		CapsulesConnectedWithNew[i] = new int*[MaxBondsForCapsule];
		for (int j = 0; j <= MaxBondsForCapsule - 1; j++)
			CapsulesConnectedWithNew[i][j] = new int[2];
	}

	for (int i = 0; i <= 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCapsuleold - 1; j++)
		{
			CapsulesConnectedWithNew[i][j][0] = CapsulesConnectedWith[i][j][0];
			CapsulesConnectedWithNew[i][j][1] = CapsulesConnectedWith[i][j][1];
		}
		for (int j = MaxBondsForCapsuleold; j <= MaxBondsForCapsule - 1; j++)
		{
			CapsulesConnectedWithNew[i][j][0] = -1;
			CapsulesConnectedWithNew[i][j][1] = -1;
		}
	}

	//delete old shorter arrays

	for (int i = 0; i <= 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCapsuleold - 1; j++)
			delete[] CapsulesConnectedWith[i][j];
		delete[] CapsulesConnectedWith[i];
	}
	delete[] CapsulesConnectedWith;

	CapsulesConnectedWith = NULL;

	// reassign
	CapsulesConnectedWith = new int**[2];
	for (int i = 0; i <= 1; i++)
	{
		CapsulesConnectedWith[i] = new int*[MaxBondsForCapsule];
		for (int j = 0; j <= MaxBondsForCapsule - 1; j++)
			CapsulesConnectedWith[i][j] = new int[2];
	}
	for (int i = 0; i <= 1; i++)
	{
		for (int j = 0; j <= MaxBondsForCapsule - 1; j++)
		{
			CapsulesConnectedWith[i][j][0] = CapsulesConnectedWithNew[i][j][0];
			CapsulesConnectedWith[i][j][1] = CapsulesConnectedWithNew[i][j][1];
		}
	}

	//delete auxiliary arrays

	for (int i = 0; i <= 1; i++)
		for (int j = 0; j <= MaxBondsForCapsule - 1; j++)
			delete[] CapsulesConnectedWithNew[i][j];
	for (int i = 0; i <= 1; i++)
		delete[] CapsulesConnectedWithNew[i];
	delete[] CapsulesConnectedWithNew;

	CapsulesConnectedWithNew = NULL;

//	cout << "\n Finished IncreaseMaxBondsForCapsule\n";
}
